@@ -84,15 +84,6 @@ type Response struct {
 	Metadata json.RawMessage `json:"metadata"`
 }
 
-func IsSnapshot(name string) bool {
-	x := strings.SplitN(name, "/", 2)
-
-	if len(x) == 2 {
-		return true
-	}
-	return false
-}
-
 func (r *Response) MetadataAsMap() (*shared.Jmap, error) {
 	ret := shared.Jmap{}
 	if err := json.Unmarshal(r.Metadata, &ret); err != nil {
@@ -232,6 +223,7 @@ func NewClient(config *Config, remote string) (*Client, error) {
 			tr := &http.Transport{
 				TLSClientConfig: tlsconfig,
 				Dial:            shared.RFC3493Dialer,
+				Proxy:           http.ProxyFromEnvironment,
 			}
 
 			c.websocketDialer = websocket.Dialer{
@@ -483,41 +475,19 @@ func (c *Client) AmTrusted() bool {
 	return auth == "trusted"
 }
 
-func (c *Client) ListContainers() ([]string, error) {
-	resp, err := c.get("containers")
+func (c *Client) ListContainers() ([]shared.ContainerInfo, error) {
+	resp, err := c.get("containers?recursion=1")
 	if err != nil {
 		return nil, err
 	}
 
-	var result []string
+	var result []shared.ContainerInfo
 
 	if err := json.Unmarshal(resp.Metadata, &result); err != nil {
 		return nil, err
 	}
 
-	names := []string{}
-
-	for _, url := range result {
-		toScan := strings.Replace(url, "/", " ", -1)
-		version := ""
-		name := ""
-		count, err := fmt.Sscanf(toScan, " %s containers %s", &version, &name)
-		if err != nil {
-			return nil, err
-		}
-
-		if count != 2 {
-			return nil, fmt.Errorf(gettext.Gettext("bad container url %s"), url)
-		}
-
-		if version != shared.APIVersion {
-			return nil, fmt.Errorf(gettext.Gettext("bad version in container url"))
-		}
-
-		names = append(names, name)
-	}
-
-	return names, nil
+	return result, nil
 }
 
 func (c *Client) CopyImage(image string, dest *Client, copy_aliases bool, aliases []string, public bool) error {
@@ -836,13 +806,13 @@ func (c *Client) UserAuthServerCert(name string) error {
 	return err
 }
 
-func (c *Client) CertificateList() (map[string]string, error) {
+func (c *Client) CertificateList() ([]string, error) {
 	raw, err := c.get("certificates")
 	if err != nil {
 		return nil, err
 	}
 
-	ret := make(map[string]string)
+	ret := []string{}
 	if err := json.Unmarshal(raw.Metadata, &ret); err != nil {
 		return nil, err
 	}
@@ -987,7 +957,7 @@ func (c *Client) Init(name string, imgremote string, image string, profiles *[]s
 		return nil, err
 	}
 
-	return resp, c.WaitForSuccess(resp.Operation)
+	return resp, nil
 }
 
 func (c *Client) LocalCopy(source string, name string, config map[string]string, profiles []string) (*Response, error) {
@@ -1028,7 +998,7 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *o
 	}
 
 	if interactive {
-		conn, err := c.websocket(resp.Operation, md.FDs[string(0)])
+		conn, err := c.websocket(resp.Operation, md.FDs["0"])
 		if err != nil {
 			return -1, err
 		}
@@ -1039,7 +1009,7 @@ func (c *Client) Exec(name string, cmd []string, env map[string]string, stdin *o
 		conns := make([]*websocket.Conn, 3)
 		dones := make([]chan bool, 3)
 		for i := 0; i < 3; i++ {
-			conns[i], err = c.websocket(resp.Operation, md.FDs[string(i)])
+			conns[i], err = c.websocket(resp.Operation, md.FDs[strconv.Itoa(i)])
 			if err != nil {
 				return -1, err
 			}
@@ -1110,10 +1080,11 @@ func (c *Client) Delete(name string) (*Response, error) {
 	return c.delete(url, nil, Async)
 }
 
-func (c *Client) ContainerStatus(name string) (*shared.ContainerState, error) {
+func (c *Client) ContainerStatus(name string, showLog bool) (*shared.ContainerState, error) {
 	ct := shared.ContainerState{}
+	query := url.Values{"log": []string{fmt.Sprintf("%v", showLog)}}
 
-	resp, err := c.get(fmt.Sprintf("containers/%s", name))
+	resp, err := c.get(fmt.Sprintf("containers/%s", name) + "?" + query.Encode())
 	if err != nil {
 		return nil, err
 	}
@@ -1190,12 +1161,13 @@ func (c *Client) MigrateTo(container string) (*Response, error) {
 	return c.post(fmt.Sprintf("containers/%s", container), body, Async)
 }
 
-func (c *Client) MigrateFrom(name string, operation string, secrets map[string]string, config map[string]string, profiles []string) (*Response, error) {
+func (c *Client) MigrateFrom(name string, operation string, secrets map[string]string, config map[string]string, profiles []string, baseImage string) (*Response, error) {
 	source := shared.Jmap{
-		"type":      "migration",
-		"mode":      "pull",
-		"operation": operation,
-		"secrets":   secrets,
+		"type":       "migration",
+		"mode":       "pull",
+		"operation":  operation,
+		"secrets":    secrets,
+		"base-image": baseImage,
 	}
 	body := shared.Jmap{
 		"source":   source,
@@ -1293,7 +1265,7 @@ func (c *Client) ListSnapshots(container string) ([]string, error) {
  * return string array representing a container's full configuration
  */
 func (c *Client) GetContainerConfig(container string) ([]string, error) {
-	st, err := c.ContainerStatus(container)
+	st, err := c.ContainerStatus(container, false)
 	var resp []string
 	if err != nil {
 		return resp, err
@@ -1312,7 +1284,7 @@ func (c *Client) GetContainerConfig(container string) ([]string, error) {
 }
 
 func (c *Client) SetContainerConfig(container, key, value string) (*Response, error) {
-	st, err := c.ContainerStatus(container)
+	st, err := c.ContainerStatus(container, false)
 	if err != nil {
 		return nil, err
 	}
@@ -1327,6 +1299,16 @@ func (c *Client) SetContainerConfig(container, key, value string) (*Response, er
 	return c.put(fmt.Sprintf("containers/%s", container), body, Async)
 }
 
+func (c *Client) UpdateContainerConfig(container string, st shared.BriefContainerState) error {
+	body := shared.Jmap{"name": container,
+		"profiles":  st.Profiles,
+		"config":    st.Config,
+		"devices":   st.Devices,
+		"ephemeral": st.Ephemeral}
+	_, err := c.put(fmt.Sprintf("containers/%s", container), body, Async)
+	return err
+}
+
 func (c *Client) ProfileCreate(p string) error {
 	body := shared.Jmap{"name": p}
 
@@ -1413,7 +1395,7 @@ func (c *Client) ListProfiles() ([]string, error) {
 }
 
 func (c *Client) ApplyProfile(container, profile string) (*Response, error) {
-	st, err := c.ContainerStatus(container)
+	st, err := c.ContainerStatus(container, false)
 	if err != nil {
 		return nil, err
 	}
@@ -1424,7 +1406,7 @@ func (c *Client) ApplyProfile(container, profile string) (*Response, error) {
 }
 
 func (c *Client) ContainerDeviceDelete(container, devname string) (*Response, error) {
-	st, err := c.ContainerStatus(container)
+	st, err := c.ContainerStatus(container, false)
 	if err != nil {
 		return nil, err
 	}
@@ -1436,7 +1418,7 @@ func (c *Client) ContainerDeviceDelete(container, devname string) (*Response, er
 }
 
 func (c *Client) ContainerDeviceAdd(container, devname, devtype string, props []string) (*Response, error) {
-	st, err := c.ContainerStatus(container)
+	st, err := c.ContainerStatus(container, false)
 	if err != nil {
 		return nil, err
 	}
@@ -1462,7 +1444,7 @@ func (c *Client) ContainerDeviceAdd(container, devname, devtype string, props []
 }
 
 func (c *Client) ContainerListDevices(container string) ([]string, error) {
-	st, err := c.ContainerStatus(container)
+	st, err := c.ContainerStatus(container, false)
 	if err != nil {
 		return nil, err
 	}
