@@ -21,56 +21,57 @@ func (c *configCmd) showByDefault() bool {
 	return true
 }
 
-var profileEditHelp string = gettext.Gettext(
-	"### This is a yaml representation of the profile.\n" +
+var configEditHelp string = gettext.Gettext(
+	"### This is a yaml representation of the configuration.\n" +
 		"### Any line starting with a '# will be ignored.\n" +
 		"###\n" +
-		"### A profile consists of a set of configuration items followed by a set of\n" +
-		"### devices.\n" +
-		"###\n" +
-		"### An example would look like:\n" +
-		"### name: onenic\n" +
+		"### A sample configuration looks like:\n" +
+		"### name: container1\n" +
+		"### profiles:\n" +
+		"### - default\n" +
 		"### config:\n" +
-		"###   raw.lxc: lxc.aa_profile=unconfined\n" +
+		"###   volatile.eth0.hwaddr: 00:16:3e:e9:f8:7f\n" +
 		"### devices:\n" +
-		"###   eth0:\n" +
-		"###     nictype: bridged\n" +
-		"###     parent: lxcbr0\n" +
-		"###     type: nic\n" +
+		"###   homedir:\n" +
+		"###     path: /extra\n" +
+		"###     source: /home/user\n" +
+		"###     type: disk\n" +
+		"### ephemeral: false\n" +
 		"###\n" +
-		"### Note that the name cannot be changed\n")
+		"### Note that the name is shown but cannot be changed\n")
 
 func (c *configCmd) usage() string {
 	return gettext.Gettext(
 		"Manage configuration.\n" +
 			"\n" +
+			"lxc config device add <container> <name> <type> [key=value]...\n" +
+			"               Add a device to a container\n" +
+			"lxc config device list <container>                List devices for container\n" +
+			"lxc config device remove <container> <name>       Remove device from container\n" +
+			"lxc config edit <container>                      Edit container configuration in external editor\n" +
 			"lxc config get <container> key                   Get configuration key\n" +
-			"lxc config device add <resource> <name> <type> [key=value]...\n" +
-			"               Add a device to a resource\n" +
-			"lxc config device list <resource>                List devices for resource\n" +
-			"lxc config device remove <resource> <name>       Remove device from resource\n" +
-			"lxc config profile list [filters]                List profiles\n" +
-			"lxc config profile create <profile>              Create profile\n" +
-			"lxc config profile delete <profile>              Delete profile\n" +
-			"lxc config profile device add <profile> <name> <type> [key=value]...\n" +
-			"               Delete profile\n" +
-			"lxc config profile edit <profile>                Edit profile in external editor\n" +
-			"lxc config profile device list <profile>\n" +
-			"lxc config profile device remove <profile> <name>\n" +
-			"lxc config profile set <profile> <key> <value>   Set profile configuration\n" +
-			"lxc config profile apply <resource> <profile>    Apply profile to container\n" +
 			"lxc config set [remote] password <newpwd>        Set admin password\n" +
 			"lxc config set <container> key [value]           Set container configuration key\n" +
 			"lxc config show <container>                      Show container configuration\n" +
 			"lxc config trust list [remote]                   List all trusted certs.\n" +
 			"lxc config trust add [remote] [certfile.crt]     Add certfile.crt to trusted hosts.\n" +
 			"lxc config trust remove [remote] [hostname|fingerprint]\n" +
-			"               Remove the cert from trusted hosts.\n")
+			"               Remove the cert from trusted hosts.\n" +
+			"\n" +
+			"Examples:\n" +
+			"To mount host's /share/c1 onto /opt in the container:\n" +
+			"\tlxc config device add container1 mntdir disk source=/share/c1 path=opt\n" +
+			"To set an lxc config value:\n" +
+			"\tlxc config set <container> raw.lxc 'lxc.aa_allow_incomplete = 1'\n")
 }
 
 func (c *configCmd) flags() {}
 
 func doSet(config *lxd.Config, args []string) error {
+	if len(args) != 4 {
+		return errArgs
+	}
+
 	// [[lxc config]] set dakara:c1 limits.memory 200000
 	remote, container := config.ParseRemoteAndContainer(args[1])
 	d, err := lxd.NewClient(config, remote)
@@ -79,12 +80,7 @@ func doSet(config *lxd.Config, args []string) error {
 	}
 
 	key := args[2]
-	var value string
-	if len(args) < 4 {
-		value = ""
-	} else {
-		value = args[3]
-	}
+	value := args[3]
 	resp, err := d.SetContainerConfig(container, key, value)
 	if err != nil {
 		return err
@@ -103,7 +99,7 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 		if len(args) < 3 {
 			return errArgs
 		}
-		return doSet(config, args)
+		return doSet(config, append(args, ""))
 
 	case "set":
 		if len(args) < 2 {
@@ -155,8 +151,8 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 				return err
 			}
 
-			for host, fingerprint := range trust {
-				fmt.Println(fmt.Sprintf("%s: %s", host, fingerprint))
+			for _, fingerprint := range trust {
+				fmt.Println(fmt.Sprintf("%s", fingerprint))
 			}
 
 			return nil
@@ -198,19 +194,6 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 				return err
 			}
 
-			toRemove := args[len(args)-1]
-			trust, err := d.CertificateList()
-			if err != nil {
-				return err
-			}
-
-			/* Try to remove by hostname first. */
-			for host, fingerprint := range trust {
-				if host == toRemove {
-					return d.CertificateRemove(fingerprint)
-				}
-			}
-
 			return d.CertificateRemove(args[len(args)-1])
 		default:
 			return fmt.Errorf(gettext.Gettext("Unkonwn config trust command %s"), args[1])
@@ -246,7 +229,7 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 			return err
 		}
 
-		resp, err := d.ContainerStatus(container)
+		resp, err := d.ContainerStatus(container, false)
 		if err != nil {
 			return err
 		}
@@ -254,56 +237,6 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 		return nil
 
 	case "profile":
-		if len(args) < 2 {
-			return errArgs
-		}
-		if args[1] == "list" {
-			return doProfileList(config, args)
-		}
-		if len(args) < 3 {
-			return errArgs
-		}
-
-		if args[1] == "device" {
-			return doProfileDevice(config, args)
-		}
-
-		remote, profile := config.ParseRemoteAndContainer(args[2])
-		client, err := lxd.NewClient(config, remote)
-		if err != nil {
-			return err
-		}
-
-		switch args[1] {
-		case "create":
-			return doProfileCreate(client, profile)
-		case "delete":
-			return doProfileDelete(client, profile)
-		case "edit":
-			return doProfileEdit(client, profile)
-		case "apply":
-			container := profile
-			switch len(args) {
-			case 3:
-				profile = ""
-			case 4:
-				profile = args[3]
-			default:
-				return errArgs
-			}
-			return doProfileApply(client, container, profile)
-		case "get":
-			return doProfileGet(client, profile, args[3:])
-		case "set":
-			return doProfileSet(client, profile, args[3:])
-		case "unset":
-			return doProfileSet(client, profile, args[3:])
-		case "copy":
-			return doProfileCopy(config, client, profile, args[3:])
-		case "show":
-			return doProfileShow(client, profile)
-		}
-
 	case "device":
 		if len(args) < 2 {
 			return errArgs
@@ -319,6 +252,19 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 			return errArgs
 		}
 
+	case "edit":
+		if len(args) != 2 {
+			return errArgs
+		}
+
+		remote, container := config.ParseRemoteAndContainer(args[1])
+		d, err := lxd.NewClient(config, remote)
+		if err != nil {
+			return err
+		}
+
+		return doConfigEdit(d, container)
+
 	default:
 		return errArgs
 	}
@@ -326,19 +272,14 @@ func (c *configCmd) run(config *lxd.Config, args []string) error {
 	return errArgs
 }
 
-func doProfileCreate(client *lxd.Client, p string) error {
-	err := client.ProfileCreate(p)
-	if err == nil {
-		fmt.Printf(gettext.Gettext("Profile %s created\n"), p)
-	}
-	return err
-}
-
-func doProfileEdit(client *lxd.Client, p string) error {
-	profile, err := client.ProfileConfig(p)
+func doConfigEdit(client *lxd.Client, cont string) error {
+	config, err := client.ContainerStatus(cont, false)
 	if err != nil {
 		return err
 	}
+
+	brief := config.BriefState()
+
 	editor := os.Getenv("VISUAL")
 	if editor == "" {
 		editor = os.Getenv("EDITOR")
@@ -346,177 +287,53 @@ func doProfileEdit(client *lxd.Client, p string) error {
 			editor = "vi"
 		}
 	}
-	data, err := yaml.Marshal(&profile)
-	f, err := ioutil.TempFile("", "lxc_profile_")
+	data, err := yaml.Marshal(&brief)
+	f, err := ioutil.TempFile("", "lxd_lxc_config_")
 	if err != nil {
 		return err
 	}
 	fname := f.Name()
-	if err = f.Chmod(0700); err != nil {
+	if err = f.Chmod(0600); err != nil {
 		f.Close()
 		os.Remove(fname)
 		return err
 	}
-	f.Write([]byte(profileEditHelp))
+	f.Write([]byte(configEditHelp))
 	f.Write(data)
 	f.Close()
 	defer os.Remove(fname)
-	cmd := exec.Command(editor, fname)
-	cmd.Stdin = os.Stdin
-	cmd.Stdout = os.Stdout
-	cmd.Stderr = os.Stderr
-	err = cmd.Run()
-	if err != nil {
-		return err
-	}
-	contents, err := ioutil.ReadFile(fname)
-	if err != nil {
-		return err
-	}
-	newdata := shared.ProfileConfig{}
-	err = yaml.Unmarshal(contents, &newdata)
-	if err != nil {
-		return err
-	}
-	err = client.PutProfile(p, newdata)
-	return err
-}
 
-func doProfileDelete(client *lxd.Client, p string) error {
-	err := client.ProfileDelete(p)
-	if err == nil {
-		fmt.Printf(gettext.Gettext("Profile %s deleted\n"), p)
+	for {
+		cmd := exec.Command(editor, fname)
+		cmd.Stdin = os.Stdin
+		cmd.Stdout = os.Stdout
+		cmd.Stderr = os.Stderr
+		err = cmd.Run()
+		if err != nil {
+			return err
+		}
+		contents, err := ioutil.ReadFile(fname)
+		if err != nil {
+			return err
+		}
+		newdata := shared.BriefContainerState{}
+		err = yaml.Unmarshal(contents, &newdata)
+		if err != nil {
+			fmt.Fprintf(os.Stderr, gettext.Gettext("YAML parse error %v\n"), err)
+			fmt.Printf("Press enter to play again ")
+			_, err := os.Stdin.Read(make([]byte, 1))
+			if err != nil {
+				return err
+			}
+
+			continue
+		}
+		err = client.UpdateContainerConfig(cont, newdata)
+		break
 	}
 	return err
 }
 
-func doProfileApply(client *lxd.Client, c string, p string) error {
-	resp, err := client.ApplyProfile(c, p)
-	if err == nil {
-		if p == "" {
-			p = "(none)"
-		}
-		fmt.Printf(gettext.Gettext("Profile %s applied to %s\n"), p, c)
-	} else {
-		return err
-	}
-	return client.WaitForSuccess(resp.Operation)
-}
-
-func doProfileShow(client *lxd.Client, p string) error {
-	resp, err := client.GetProfileConfig(p)
-	if err != nil {
-		return err
-	}
-	for k, v := range resp {
-		fmt.Printf("%s = %s\n", k, v)
-	}
-
-	dresp, err := client.ProfileListDevices(p)
-	if err != nil {
-		return err
-	}
-	fmt.Printf("%s\n", strings.Join(dresp, "\n"))
-
-	return nil
-}
-
-func doProfileCopy(config *lxd.Config, client *lxd.Client, p string, args []string) error {
-	if len(args) != 1 {
-		return errArgs
-	}
-	remote, newname := config.ParseRemoteAndContainer(args[0])
-	if newname == "" {
-		newname = p
-	}
-
-	dest, err := lxd.NewClient(config, remote)
-	if err != nil {
-		return err
-	}
-
-	return client.ProfileCopy(p, newname, dest)
-}
-
-func doProfileDevice(config *lxd.Config, args []string) error {
-	// profile device add b1 eth0 nic type=bridged
-	// profile device list b1
-	// profile device remove b1 eth0
-	if len(args) < 4 {
-		return errArgs
-	}
-	switch args[2] {
-	case "add":
-		return deviceAdd(config, "profile", args[1:])
-	case "remove":
-		return deviceRm(config, "profile", args[1:])
-	case "list":
-		return deviceList(config, "profile", args[1:])
-	default:
-		return errArgs
-	}
-}
-
-func doProfileGet(client *lxd.Client, p string, args []string) error {
-	// we shifted @args so so it should read "<key>"
-	if len(args) != 1 {
-		return errArgs
-	}
-
-	resp, err := client.GetProfileConfig(p)
-	if err != nil {
-		return err
-	}
-	for k, v := range resp {
-		if k == args[0] {
-			fmt.Printf("%s\n", v)
-		}
-	}
-	return nil
-}
-
-func doProfileSet(client *lxd.Client, p string, args []string) error {
-	// we shifted @args so so it should read "<key> [<value>]"
-	if len(args) < 1 {
-		return errArgs
-	}
-
-	key := args[0]
-	var value string
-	if len(args) < 2 {
-		value = ""
-	} else {
-		value = args[1]
-	}
-	err := client.SetProfileConfigItem(p, key, value)
-	return err
-}
-
-func doProfileList(config *lxd.Config, args []string) error {
-	var remote string
-	if len(args) > 2 {
-		var name string
-		remote, name = config.ParseRemoteAndContainer(args[2])
-		if name != "" {
-			return fmt.Errorf(gettext.Gettext("Cannot provide container name to list"))
-		}
-	} else {
-		remote = config.DefaultRemote
-	}
-
-	client, err := lxd.NewClient(config, remote)
-	if err != nil {
-		return err
-	}
-
-	profiles, err := client.ListProfiles()
-	if err != nil {
-		return err
-	}
-	fmt.Printf("%s\n", strings.Join(profiles, "\n"))
-	return nil
-}
-
 func deviceAdd(config *lxd.Config, which string, args []string) error {
 	if len(args) < 5 {
 		return errArgs
