@@ -46,19 +46,31 @@ func copyContainer(config *lxd.Config, sourceResource string, destResource strin
 		return err
 	}
 
-	status, err := source.ContainerStatus(sourceName)
-	if err != nil {
-		return err
-	}
+	status := shared.ContainerState{}
 
-	if status.State() == lxc.RUNNING && sourceName != destName {
-		return fmt.Errorf(gettext.Gettext("changing hostname of running containers not supported"))
-	}
+	// TODO: presumably we want to do this for copying snapshots too? We
+	// need to think a bit more about how we track the baseImage in the
+	// face of LVM and snapshots in general; this will probably make more
+	// sense once that work is done.
+	baseImage := ""
 
-	if !keepVolatile {
-		for k := range status.Config {
-			if strings.HasPrefix(k, "volatile") {
-				delete(status.Config, k)
+	if !shared.IsSnapshot(sourceName) {
+		status, err := source.ContainerStatus(sourceName, false)
+		if err != nil {
+			return err
+		}
+
+		baseImage = status.Config["volatile.baseImage"]
+
+		if status.State() == lxc.RUNNING && sourceName != destName {
+			return fmt.Errorf(gettext.Gettext("changing hostname of running containers not supported"))
+		}
+
+		if !keepVolatile {
+			for k := range status.Config {
+				if strings.HasPrefix(k, "volatile") {
+					delete(status.Config, k)
+				}
 			}
 		}
 	}
@@ -106,7 +118,7 @@ func copyContainer(config *lxd.Config, sourceResource string, destResource strin
 		}
 
 		url := source.BaseWSURL + path.Join(to.Operation, "websocket")
-		migration, err := dest.MigrateFrom(sourceName, url, secrets, status.Config, status.Profiles)
+		migration, err := dest.MigrateFrom(sourceName, url, secrets, status.Config, status.Profiles, baseImage)
 		if err != nil {
 			return err
 		}
