@@ -19,7 +19,7 @@ func (c *deleteCmd) usage() string {
 	return gettext.Gettext(
 		"Delete a container or container snapshot.\n" +
 			"\n" +
-			"Destroy a resource (e.g. container) and any attached data (configuration,\n" +
+			"Destroy a container (or snapshot) and any attached data (configuration,\n" +
 			"snapshots, ...).\n")
 }
 
@@ -31,15 +31,7 @@ func doDelete(d *lxd.Client, name string) error {
 		return err
 	}
 
-	op, err := d.WaitFor(resp.Operation)
-	if err != nil {
-		return err
-	}
-
-	if op.StatusCode == shared.Success {
-		return nil
-	}
-	return fmt.Errorf(gettext.Gettext("Operation %s"), op.Status)
+	return d.WaitForSuccess(resp.Operation)
 }
 
 func (c *deleteCmd) run(config *lxd.Config, args []string) error {
@@ -54,7 +46,7 @@ func (c *deleteCmd) run(config *lxd.Config, args []string) error {
 		return err
 	}
 
-	ct, err := d.ContainerStatus(name)
+	ct, err := d.ContainerStatus(name, false)
 
 	if err != nil {
 		// Could be a snapshot
@@ -75,10 +67,10 @@ func (c *deleteCmd) run(config *lxd.Config, args []string) error {
 		if op.StatusCode == shared.Failure {
 			return fmt.Errorf(gettext.Gettext("Stopping container failed!"))
 		}
-	}
 
-	if ct.Ephemeral == true {
-		return nil
+		if ct.Ephemeral == true {
+			return nil
+		}
 	}
 
 	return doDelete(d, name)
