@@ -57,7 +57,7 @@ func (c *execCmd) run(config *lxd.Config, args []string) error {
 		return err
 	}
 
-	env := map[string]string{"HOME": "/root"}
+	env := map[string]string{"HOME": "/root", "USER": "root"}
 	myEnv := os.Environ()
 	for _, ent := range myEnv {
 		if strings.HasPrefix(ent, "TERM=") {
