@@ -2,7 +2,6 @@ package main
 
 import (
 	"fmt"
-	"net/http"
 	"os"
 	"strings"
 
@@ -92,29 +91,21 @@ func shouldShow(filters []string, state *shared.ContainerState) bool {
 	return true
 }
 
-func listContainers(d *lxd.Client, cts []string, filters []string, showsnaps bool) error {
+func listContainers(client *lxd.Client, cinfos []shared.ContainerInfo, filters []string, listsnaps bool) error {
 	data := [][]string{}
 
-	for _, ct := range cts {
-		// get more information
-		c, err := d.ContainerStatus(ct)
-		d := []string{}
-		if err == nil {
-			d = []string{ct, c.Status.State}
-		} else if err == lxd.LXDErrors[http.StatusNotFound] {
-			return fmt.Errorf(gettext.Gettext("Container not found"))
-		} else {
-			return err
-		}
+	for _, cinfo := range cinfos {
+		cstate := cinfo.State
+		d := []string{cstate.Name, cstate.Status.State}
 
-		if !shouldShow(filters, c) {
+		if !shouldShow(filters, &cstate) {
 			continue
 		}
 
-		if c.Status.State == "RUNNING" {
+		if cstate.Status.State == "RUNNING" {
 			ipv4s := []string{}
 			ipv6s := []string{}
-			for _, ip := range c.Status.Ips {
+			for _, ip := range cstate.Status.Ips {
 				if ip.Interface == "lo" {
 					continue
 				}
@@ -133,16 +124,20 @@ func listContainers(d *lxd.Client, cts []string, filters []string, showsnaps boo
 			d = append(d, "")
 			d = append(d, "")
 		}
-		if c.Ephemeral {
+		if cstate.Ephemeral {
 			d = append(d, "YES")
 		} else {
 			d = append(d, "NO")
 		}
+		// List snapshots
+		csnaps := cinfo.Snaps
+		d = append(d, fmt.Sprintf("%d", len(csnaps)))
+
 		data = append(data, d)
 	}
 
 	table := tablewriter.NewWriter(os.Stdout)
-	table.SetHeader([]string{"NAME", "STATE", "IPV4", "IPV6", "EPHEMERAL"})
+	table.SetHeader([]string{"NAME", "STATE", "IPV4", "IPV6", "EPHEMERAL", "SNAPSHOTS"})
 
 	for _, v := range data {
 		table.Append(v)
@@ -150,14 +145,10 @@ func listContainers(d *lxd.Client, cts []string, filters []string, showsnaps boo
 
 	table.Render() // Send output
 
-	if showsnaps {
-		cName := cts[0]
+	if listsnaps && len(cinfos) == 1 {
+		csnaps := cinfos[0].Snaps
 		first_snapshot := true
-		snaps, err := d.ListSnapshots(cName)
-		if err != nil {
-			return nil
-		}
-		for _, snap := range snaps {
+		for _, snap := range csnaps {
 			if first_snapshot {
 				fmt.Printf("Snapshots:\n")
 			}
@@ -170,13 +161,12 @@ func listContainers(d *lxd.Client, cts []string, filters []string, showsnaps boo
 
 func (c *listCmd) run(config *lxd.Config, args []string) error {
 	var remote string
-	var name string
+	name := ""
 
 	filters := []string{}
 
 	if len(args) == 0 {
 		remote = config.DefaultRemote
-		name = ""
 	} else {
 		filters = args
 		if strings.Contains(args[0], ":") {
@@ -184,7 +174,7 @@ func (c *listCmd) run(config *lxd.Config, args []string) error {
 			filters = args[1:]
 		} else if !strings.Contains(args[0], "=") {
 			remote = config.DefaultRemote
-			name = ""
+			name = args[0]
 		}
 	}
 
@@ -193,15 +183,28 @@ func (c *listCmd) run(config *lxd.Config, args []string) error {
 		return err
 	}
 
-	var cts []string
+	var cts []shared.ContainerInfo
 	if name == "" {
-		cts, err = d.ListContainers()
+		ctslist, err := d.ListContainers()
 		if err != nil {
 			return err
 		}
+		cts = ctslist
 	} else {
-		cts = []string{name}
+		//Get direct, only Once
+		cstatus, err := d.ContainerStatus(name, false)
+		if err != nil {
+			return err
+		}
+		csnaps, err := d.ListSnapshots(name)
+		if err != nil {
+			return err
+		}
+		c := shared.ContainerInfo{State: *cstatus,
+			Snaps: csnaps}
+		cts = []shared.ContainerInfo{c}
+
 	}
 
-	return listContainers(d, cts, filters, len(cts) == 1)
+	return listContainers(d, cts, filters, name != "")
 }
