@@ -15,6 +15,7 @@ import (
 	"path"
 	"path/filepath"
 	"regexp"
+	"runtime"
 	"strconv"
 	"strings"
 	"syscall"
@@ -36,9 +37,14 @@ const (
 )
 
 func containersGet(d *Daemon, r *http.Request) Response {
-	slept := time.Millisecond * 0
+	recursion_str := r.FormValue("recursion")
+	recursion, err := strconv.Atoi(recursion_str)
+	if err != nil {
+		recursion = 0
+	}
+
 	for {
-		result, err := doContainersGet(d)
+		result, err := doContainersGet(d, recursion)
 		if err == nil {
 			return SyncResponse(true, result)
 		}
@@ -46,39 +52,86 @@ func containersGet(d *Daemon, r *http.Request) Response {
 			shared.Debugf("DBERR: containersGet: error %q\n", err)
 			return InternalError(err)
 		}
-		if slept == 30*time.Second {
-			shared.Debugf("DBERR: containersGet: DB Locked for 30 seconds\n")
-			return InternalError(err)
-		}
-		time.Sleep(100 * time.Millisecond)
-		slept = slept + 100*time.Millisecond
+		// 1 s may seem drastic, but we really don't want to thrash
+		// perhaps we should use a random amount
+		shared.Debugf("DBERR: containersGet, db is locked\n")
+		shared.PrintStack()
+		time.Sleep(1 * time.Second)
 	}
 }
 
-func doContainersGet(d *Daemon) ([]string, error) {
+func doContainerGet(d *Daemon, cname string) (shared.ContainerInfo, Response) {
+	_, err := dbGetContainerId(d.db, cname)
+	if err != nil {
+		return shared.ContainerInfo{}, SmartError(err)
+	}
+
+	c, err := newLxdContainer(cname, d)
+	if err != nil {
+		return shared.ContainerInfo{}, SmartError(err)
+	}
+
+	var name string
+	regexp := fmt.Sprintf("%s/", cname)
+	length := len(regexp)
+	q := "SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?"
+	inargs := []interface{}{cTypeSnapshot, length, regexp}
+	outfmt := []interface{}{name}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	if err != nil {
+		return shared.ContainerInfo{}, SmartError(err)
+	}
+
+	var body []string
+
+	for _, r := range results {
+		name = r[0].(string)
+
+		url := fmt.Sprintf("/%s/containers/%s/snapshots/%s", shared.APIVersion, cname, name)
+		body = append(body, url)
+	}
+
+	cts, err := c.RenderState()
+	if err != nil {
+		return shared.ContainerInfo{}, SmartError(err)
+	}
+
+	containerinfo := shared.ContainerInfo{State: *cts,
+		Snaps: body}
+
+	return containerinfo, nil
+}
+
+func doContainersGet(d *Daemon, recursion int) (interface{}, error) {
 	q := fmt.Sprintf("SELECT name FROM containers WHERE type=?")
-	rows, err := shared.DbQuery(d.db, q, cTypeRegular)
+	inargs := []interface{}{cTypeRegular}
+	var container string
+	outfmt := []interface{}{container}
+	result, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	result_string := make([]string, 0)
+	result_map := make([]shared.ContainerInfo, 0)
 	if err != nil {
 		return []string{}, err
 	}
-	defer rows.Close()
-
-	result := []string{}
-	for rows.Next() {
-		container := ""
-		err = rows.Scan(&container)
-		if err != nil {
-			return []string{}, err
+	for _, r := range result {
+		container := string(r[0].(string))
+		if recursion == 0 {
+			url := fmt.Sprintf("/%s/containers/%s", shared.APIVersion, container)
+			result_string = append(result_string, url)
+		} else {
+			container, response := doContainerGet(d, container)
+			if response != nil {
+				continue
+			}
+			result_map = append(result_map, container)
 		}
-
-		result = append(result, fmt.Sprintf("/%s/containers/%s", shared.APIVersion, container))
-	}
-	err = rows.Err()
-	if err != nil {
-		return []string{}, err
 	}
 
-	return result, nil
+	if recursion == 0 {
+		return result_string, nil
+	} else {
+		return result_map, nil
+	}
 }
 
 type containerImageSource struct {
@@ -90,6 +143,15 @@ type containerImageSource struct {
 	Server      string `json:"server"`
 	Secret      string `json:"secret"`
 
+	/*
+	 * for "migration" and "copy" types, as an optimization users can
+	 * provide an image hash to extract before the filesystem is rsync'd,
+	 * potentially cutting down filesystem transfer time. LXD will not go
+	 * and fetch this image, it will simply use it if it exists in the
+	 * image store.
+	 */
+	BaseImage string `json:"base-image"`
+
 	/* for "migration" type */
 	Mode       string            `json:"mode"`
 	Operation  string            `json:"operation"`
@@ -107,12 +169,65 @@ type containerPostReq struct {
 	Ephemeral bool                 `json:"ephemeral"`
 }
 
+func containerWatchEphemeral(c *lxdContainer) {
+	go func() {
+		c.c.Wait(lxc.STOPPED, -1*time.Second)
+		c.c.Wait(lxc.RUNNING, 1*time.Second)
+		c.c.Wait(lxc.STOPPED, -1*time.Second)
+
+		_, err := dbGetContainerId(c.daemon.db, c.name)
+		if err != nil {
+			return
+		}
+
+		removeContainer(c.daemon, c.name)
+	}()
+}
+
+func containersWatch(d *Daemon) error {
+	q := fmt.Sprintf("SELECT name FROM containers WHERE type=?")
+	inargs := []interface{}{cTypeRegular}
+	var name string
+	outfmt := []interface{}{name}
+
+	result, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
+	if err != nil {
+		return err
+	}
+
+	for _, r := range result {
+		container, err := newLxdContainer(string(r[0].(string)), d)
+		if err != nil {
+			return err
+		}
+
+		if container.ephemeral == true && container.c.State() != lxc.STOPPED {
+			containerWatchEphemeral(container)
+		}
+	}
+
+	/*
+	 * force collect the containers we created above; see comment in
+	 * daemon.go:createCmd.
+	 */
+	runtime.GC()
+
+	return nil
+}
+
 func createFromImage(d *Daemon, req *containerPostReq) Response {
-	var uuid string
+	var hash string
 	var err error
+	var run func() shared.OperationResult
+
+	backing_fs, err := shared.GetFilesystem(d.lxcpath)
+	if err != nil {
+		return InternalError(err)
+	}
+
 	if req.Source.Alias != "" {
 		if req.Source.Mode == "pull" && req.Source.Server != "" {
-			uuid, err = remoteGetImageFingerprint(d, req.Source.Server, req.Source.Alias)
+			hash, err = remoteGetImageFingerprint(d, req.Source.Server, req.Source.Alias)
 			if err != nil {
 				return InternalError(err)
 			}
@@ -121,52 +236,89 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 			if err != nil {
 				return InternalError(err)
 			}
-			uuid, err = dbImageGetById(d, iId)
+			hash, err = dbImageGetById(d, iId)
 			if err != nil {
 				return InternalError(fmt.Errorf("Stale alias"))
 			}
 		}
 	} else if req.Source.Fingerprint != "" {
-		uuid = req.Source.Fingerprint
+		hash = req.Source.Fingerprint
 	} else {
 		return BadRequest(fmt.Errorf("must specify one of alias or fingerprint for init from image"))
 	}
 
 	if req.Source.Server != "" {
-		err := ensureLocalImage(d, req.Source.Server, uuid, req.Source.Secret)
+		err := ensureLocalImage(d, req.Source.Server, hash, req.Source.Secret)
 		if err != nil {
 			return InternalError(err)
 		}
 	}
 
-	imgInfo, err := dbImageGet(d, uuid, false)
+	imgInfo, err := dbImageGet(d, hash, false)
 	if err != nil {
 		return SmartError(err)
 	}
-	uuid = imgInfo.Fingerprint
+	hash = imgInfo.Fingerprint
 
 	dpath := shared.VarPath("lxc", req.Name)
 	if shared.PathExists(dpath) {
 		return InternalError(fmt.Errorf("Container exists"))
 	}
 
-	rootfsPath := fmt.Sprintf("%s/rootfs", dpath)
-	err = os.MkdirAll(rootfsPath, 0700)
-	if err != nil {
-		return InternalError(fmt.Errorf("Error creating rootfs directory"))
+	name := req.Name
+
+	args := DbCreateContainerArgs{
+		d:         d,
+		name:      name,
+		ctype:     cTypeRegular,
+		config:    req.Config,
+		profiles:  req.Profiles,
+		ephem:     req.Ephemeral,
+		baseImage: hash,
 	}
 
-	name := req.Name
-	_, err = dbCreateContainer(d, name, cTypeRegular, req.Config, req.Profiles, req.Ephemeral)
+	_, err = dbCreateContainer(args)
 	if err != nil {
 		removeContainerPath(d, name)
 		return SmartError(err)
 	}
 
-	/*
-	 * extract the rootfs asynchronously
-	 */
-	run := shared.OperationWrap(func() error { return extractShiftRootfs(uuid, name, d) })
+	c, err := newLxdContainer(name, d)
+	if err != nil {
+		return SmartError(err)
+	}
+
+	if backing_fs == "btrfs" && shared.PathExists(fmt.Sprintf("%s.btrfs", shared.VarPath("images", hash))) {
+		run = shared.OperationWrap(func() error {
+			if err := btrfsCopyImage(hash, name, d); err != nil {
+				return err
+			}
+
+			if !c.isPrivileged() {
+				return shiftRootfs(name, d)
+			}
+
+			return nil
+		})
+	} else {
+		rootfsPath := fmt.Sprintf("%s/rootfs", dpath)
+		err = os.MkdirAll(rootfsPath, 0700)
+		if err != nil {
+			return InternalError(fmt.Errorf("Error creating rootfs directory"))
+		}
+
+		run = shared.OperationWrap(func() error {
+			if err := extractRootfs(hash, name, d); err != nil {
+				return err
+			}
+
+			if !c.isPrivileged() {
+				return shiftRootfs(name, d)
+			}
+
+			return nil
+		})
+	}
 
 	resources := make(map[string][]string)
 	resources["containers"] = []string{req.Name}
@@ -175,8 +327,16 @@ func createFromImage(d *Daemon, req *containerPostReq) Response {
 }
 
 func createFromNone(d *Daemon, req *containerPostReq) Response {
+	args := DbCreateContainerArgs{
+		d:        d,
+		name:     req.Name,
+		ctype:    cTypeRegular,
+		config:   req.Config,
+		profiles: req.Profiles,
+		ephem:    req.Ephemeral,
+	}
 
-	_, err := dbCreateContainer(d, req.Name, cTypeRegular, req.Config, req.Profiles, req.Ephemeral)
+	_, err := dbCreateContainer(args)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -190,13 +350,44 @@ func createFromNone(d *Daemon, req *containerPostReq) Response {
 	return &asyncResponse{run: run, resources: resources}
 }
 
+func extractShiftIfExists(d *Daemon, c *lxdContainer, hash string, name string) error {
+	if hash == "" {
+		return nil
+	}
+
+	_, err := dbImageGet(d, hash, false)
+	if err == nil {
+		if err := extractRootfs(hash, name, d); err != nil {
+			return err
+		}
+
+		if !c.isPrivileged() {
+			if err := shiftRootfs(name, d); err != nil {
+				return err
+			}
+		}
+	}
+
+	return nil
+}
+
 func createFromMigration(d *Daemon, req *containerPostReq) Response {
 
 	if req.Source.Mode != "pull" {
 		return NotImplemented
 	}
 
-	_, err := dbCreateContainer(d, req.Name, cTypeRegular, req.Config, req.Profiles, req.Ephemeral)
+	createArgs := DbCreateContainerArgs{
+		d:         d,
+		name:      req.Name,
+		ctype:     cTypeRegular,
+		config:    req.Config,
+		profiles:  req.Profiles,
+		ephem:     req.Ephemeral,
+		baseImage: req.Source.BaseImage,
+	}
+
+	_, err := dbCreateContainer(createArgs)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -215,6 +406,11 @@ func createFromMigration(d *Daemon, req *containerPostReq) Response {
 		return InternalError(err)
 	}
 
+	if err := extractShiftIfExists(d, c, req.Source.BaseImage, req.Name); err != nil {
+		removeContainer(d, req.Name)
+		return InternalError(err)
+	}
+
 	config, err := shared.GetTLSConfig(d.certf, d.keyf)
 	if err != nil {
 		removeContainer(d, req.Name)
@@ -255,12 +451,38 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 	}
 
 	// Make sure the source exists.
-	_, err := newLxdContainer(req.Source.Source, d)
+	source, err := newLxdContainer(req.Source.Source, d)
 	if err != nil {
 		return SmartError(err)
 	}
 
-	_, err = dbCreateContainer(d, req.Name, cTypeRegular, req.Config, req.Profiles, req.Ephemeral)
+	if req.Config == nil {
+		config := make(map[string]string)
+		for key, value := range source.config {
+			if key[0:8] == "volatile" {
+				shared.Debugf("skipping: %s\n", key)
+				continue
+			}
+			req.Config[key] = value
+		}
+		req.Config = config
+	}
+
+	if req.Profiles == nil {
+		req.Profiles = source.profiles
+	}
+
+	args := DbCreateContainerArgs{
+		d:         d,
+		name:      req.Name,
+		ctype:     cTypeRegular,
+		config:    req.Config,
+		profiles:  req.Profiles,
+		ephem:     req.Ephemeral,
+		baseImage: req.Source.BaseImage,
+	}
+
+	_, err = dbCreateContainer(args)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -271,11 +493,43 @@ func createFromCopy(d *Daemon, req *containerPostReq) Response {
 		return InternalError(err)
 	}
 
-	oldPath := migration.AddSlash(shared.VarPath("lxc", req.Source.Source, "rootfs"))
+	if err := extractShiftIfExists(d, source, req.Source.BaseImage, req.Name); err != nil {
+		removeContainer(d, req.Name)
+		return InternalError(err)
+	}
+
+	var oldPath string
+	if shared.IsSnapshot(req.Source.Source) {
+		snappieces := strings.SplitN(req.Source.Source, "/", 2)
+		oldPath = migration.AddSlash(shared.VarPath("lxc",
+			snappieces[0],
+			"snapshots",
+			snappieces[1],
+			"rootfs"))
+	} else {
+		oldPath = migration.AddSlash(shared.VarPath("lxc", req.Source.Source, "rootfs"))
+	}
 	newPath := fmt.Sprintf("%s/%s", dpath, "rootfs")
 	run := func() shared.OperationResult {
-		err := exec.Command("rsync", "-a", "--devices", oldPath, newPath).Run()
-		return shared.OperationError(err)
+		output, err := exec.Command("rsync", "-a", "--devices", oldPath, newPath).CombinedOutput()
+
+		if err == nil && !source.isPrivileged() {
+			err = setUnprivUserAcl(d, dpath)
+			if err != nil {
+				shared.Debugf("Error adding acl for container root: falling back to chmod\n")
+				output, err := exec.Command("chmod", "+x", dpath).CombinedOutput()
+				if err != nil {
+					shared.Debugf("Error chmoding the container root\n")
+					shared.Debugf(string(output))
+					return shared.OperationError(err)
+				}
+			}
+		} else {
+			shared.Debugf("rsync failed:\n%s", output)
+			return shared.OperationError(err)
+		}
+
+		return shared.OperationError(nil)
 	}
 
 	resources := make(map[string][]string)
@@ -316,20 +570,57 @@ func containersPost(d *Daemon, r *http.Request) Response {
 
 }
 
-func removeContainerPath(d *Daemon, name string) {
+func removeContainerPath(d *Daemon, name string) error {
 	cpath := shared.VarPath("lxc", name)
-	err := os.RemoveAll(cpath)
+
+	backing_fs, err := shared.GetFilesystem(cpath)
+	if err != nil {
+		if os.IsNotExist(err) {
+			return nil
+		}
+
+		shared.Debugf("Error cleaning up %s: %s\n", cpath, err)
+		return err
+	}
+
+	if backing_fs == "btrfs" {
+		exec.Command("btrfs", "subvolume", "delete", cpath).Run()
+	}
+
+	err = os.RemoveAll(cpath)
 	if err != nil {
 		shared.Debugf("Error cleaning up %s: %s\n", cpath, err)
+		return err
 	}
+
+	return nil
 }
 
-func removeContainer(d *Daemon, name string) {
-	removeContainerPath(d, name)
-	dbRemoveContainer(d, name)
+func removeContainer(d *Daemon, name string) error {
+	if err := containerDeleteSnapshots(d, name); err != nil {
+		return err
+	}
+
+	if err := removeContainerPath(d, name); err != nil {
+		return err
+	}
+
+	if err := dbRemoveContainer(d, name); err != nil {
+		return err
+	}
+
+	return nil
 }
 
-func extractShiftRootfs(uuid string, name string, d *Daemon) error {
+func btrfsCopyImage(hash string, name string, d *Daemon) error {
+	dpath := shared.VarPath("lxc", name)
+	imagefile := shared.VarPath("images", hash)
+	subvol := fmt.Sprintf("%s.btrfs", imagefile)
+
+	return exec.Command("btrfs", "subvolume", "snapshot", subvol, dpath).Run()
+}
+
+func extractRootfs(hash string, name string, d *Daemon) error {
 	/*
 	 * We want to use archive/tar for this, but that doesn't appear
 	 * to be working for us (see lxd/images.go)
@@ -338,7 +629,7 @@ func extractShiftRootfs(uuid string, name string, d *Daemon) error {
 	 * extract that under /var/lib/lxd/lxc/container/rootfs/
 	 */
 	dpath := shared.VarPath("lxc", name)
-	imagefile := shared.VarPath("images", uuid)
+	imagefile := shared.VarPath("images", hash)
 
 	compression, _, err := detectCompression(imagefile)
 	if err != nil {
@@ -369,8 +660,13 @@ func extractShiftRootfs(uuid string, name string, d *Daemon) error {
 		return err
 	}
 
+	return nil
+}
+
+func shiftRootfs(name string, d *Daemon) error {
+	dpath := shared.VarPath("lxc", name)
 	rpath := shared.VarPath("lxc", name, "rootfs")
-	err = d.idMap.ShiftRootfs(rpath)
+	err := d.idMap.ShiftRootfs(rpath)
 	if err != nil {
 		shared.Debugf("Shift of rootfs %s failed: %s\n", rpath, err)
 		removeContainer(d, name)
@@ -378,17 +674,32 @@ func extractShiftRootfs(uuid string, name string, d *Daemon) error {
 	}
 
 	/* Set an acl so the container root can descend the container dir */
-	acl := fmt.Sprintf("%d:rx", d.idMap.Uidmin)
-	_, err = exec.Command("setfacl", "-m", acl, dpath).Output()
+	err = setUnprivUserAcl(d, dpath)
 	if err != nil {
-		shared.Debugf("Error adding acl for container root: start will likely fail\n")
+		shared.Debugf("Error adding acl for container root: falling back to chmod\n")
+		output, err := exec.Command("chmod", "+x", dpath).CombinedOutput()
+		if err != nil {
+			shared.Debugf("Error chmoding the container root\n")
+			shared.Debugf(string(output))
+			return err
+		}
 	}
 
 	return nil
 }
 
-func dbRemoveContainer(d *Daemon, name string) {
-	_, _ = shared.DbExec(d.db, "DELETE FROM containers WHERE name=?", name)
+func setUnprivUserAcl(d *Daemon, dpath string) error {
+	acl := fmt.Sprintf("%d:rx", d.idMap.Uidmin)
+	output, err := exec.Command("setfacl", "-m", acl, dpath).CombinedOutput()
+	if err != nil {
+		shared.Debugf("setfacl failed:\n%s", output)
+	}
+	return err
+}
+
+func dbRemoveContainer(d *Daemon, name string) error {
+	_, err := shared.DbExec(d.db, "DELETE FROM containers WHERE name=?", name)
+	return err
 }
 
 func dbGetContainerId(db *sql.DB, name string) (int, error) {
@@ -400,34 +711,52 @@ func dbGetContainerId(db *sql.DB, name string) (int, error) {
 	return id, err
 }
 
-func dbCreateContainer(d *Daemon, name string, ctype containerType, config map[string]string, profiles []string, ephem bool) (int, error) {
-	id, err := dbGetContainerId(d.db, name)
+type DbCreateContainerArgs struct {
+	d         *Daemon
+	name      string
+	ctype     containerType
+	config    map[string]string
+	profiles  []string
+	ephem     bool
+	baseImage string
+}
+
+func dbCreateContainer(args DbCreateContainerArgs) (int, error) {
+	id, err := dbGetContainerId(args.d.db, args.name)
 	if err == nil {
 		return 0, DbErrAlreadyDefined
 	}
 
-	if profiles == nil {
-		profiles = []string{"default"}
+	if args.profiles == nil {
+		args.profiles = []string{"default"}
 	}
 
-	tx, err := d.db.Begin()
+	if args.baseImage != "" {
+		if args.config == nil {
+			args.config = map[string]string{}
+		}
+
+		args.config["volatile.baseImage"] = args.baseImage
+	}
+
+	tx, err := shared.DbBegin(args.d.db)
 	if err != nil {
 		return 0, err
 	}
 	ephem_int := 0
-	if ephem == true {
+	if args.ephem == true {
 		ephem_int = 1
 	}
 
 	str := fmt.Sprintf("INSERT INTO containers (name, architecture, type, ephemeral) VALUES (?, 1, %d, %d)",
-		ctype, ephem_int)
+		args.ctype, ephem_int)
 	stmt, err := tx.Prepare(str)
 	if err != nil {
 		tx.Rollback()
 		return 0, err
 	}
 	defer stmt.Close()
-	result, err := stmt.Exec(name)
+	result, err := stmt.Exec(args.name)
 	if err != nil {
 		tx.Rollback()
 		return 0, err
@@ -436,16 +765,16 @@ func dbCreateContainer(d *Daemon, name string, ctype containerType, config map[s
 	id64, err := result.LastInsertId()
 	if err != nil {
 		tx.Rollback()
-		return 0, fmt.Errorf("Error inserting %s into database", name)
+		return 0, fmt.Errorf("Error inserting %s into database", args.name)
 	}
 	// TODO: is this really int64? we should fix it everywhere if so
 	id = int(id64)
-	if err := dbInsertContainerConfig(tx, id, config); err != nil {
+	if err := dbInsertContainerConfig(tx, id, args.config); err != nil {
 		tx.Rollback()
 		return 0, err
 	}
 
-	if err := dbInsertProfiles(tx, id, profiles); err != nil {
+	if err := dbInsertProfiles(tx, id, args.profiles); err != nil {
 		tx.Rollback()
 		return 0, err
 	}
@@ -467,41 +796,72 @@ func containerGet(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	return SyncResponse(true, c.RenderState())
+	state, err := c.RenderState()
+	if err != nil {
+		return InternalError(err)
+	}
+
+	targetPath := r.FormValue("log")
+	if strings.ToLower(targetPath) == "true" || targetPath == "1" {
+		fname := c.c.LogFile()
+
+		f, err := os.Open(fname)
+		if err != nil {
+			return InternalError(err)
+		}
+		defer f.Close()
+
+		log, err := shared.ReadLastNLines(f, 100)
+		if err != nil {
+			return InternalError(err)
+		}
+		state.Log = log
+	}
+
+	return SyncResponse(true, state)
 }
 
-func containerDeleteSnapshots(d *Daemon, cname string) []string {
+func containerDeleteSnapshots(d *Daemon, cname string) error {
 	prefix := fmt.Sprintf("%s/", cname)
 	length := len(prefix)
-	rows, err := shared.DbQuery(d.db, "SELECT name, id FROM containers WHERE type=? AND SUBSTR(name,1,?)=?",
-		cTypeSnapshot, length, prefix)
+	q := "SELECT name, id FROM containers WHERE type=? AND SUBSTR(name,1,?)=?"
+	var id int
+	var sname string
+	inargs := []interface{}{cTypeSnapshot, length, prefix}
+	outfmt := []interface{}{sname, id}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
-		return nil
+		return err
 	}
 
-	var results []string
 	var ids []int
 
-	for rows.Next() {
-		var id int
-		var sname string
-		err = rows.Scan(&sname, &id)
-		if err != nil {
-			fmt.Printf("DBERR: containerDeleteSnapshots: scan returned error %q\n", err)
-		}
+	backing_fs, err := shared.GetFilesystem(shared.VarPath("lxc", cname))
+	if err != nil && !os.IsNotExist(err) {
+		shared.Debugf("Error cleaning up snapshots: %s\n", err)
+		return err
+	}
+
+	for _, r := range results {
+		sname = r[0].(string)
+		id = r[1].(int)
 		ids = append(ids, id)
 		cdir := shared.VarPath("lxc", cname, "snapshots", sname)
-		results = append(results, cdir)
+
+		if backing_fs == "btrfs" {
+			exec.Command("btrfs", "subvolume", "delete", cdir).Run()
+		}
+		os.RemoveAll(cdir)
 	}
-	err = rows.Err()
-	if err != nil {
-		fmt.Printf("DBERR: containerDeleteSnapshots: Err returned an error %q\n", err)
-	}
-	rows.Close()
+
 	for _, id := range ids {
-		_, _ = shared.DbExec(d.db, "DELETE FROM containers WHERE id=?", id)
+		_, err = shared.DbExec(d.db, "DELETE FROM containers WHERE id=?", id)
+		if err != nil {
+			return err
+		}
 	}
-	return results
+
+	return nil
 }
 
 type containerConfigReq struct {
@@ -540,12 +900,18 @@ func dbInsertContainerConfig(tx *sql.Tx, id int, config map[string]string) error
 	str := "INSERT INTO containers_config (container_id, key, value) values (?, ?, ?)"
 	stmt, err := tx.Prepare(str)
 	if err != nil {
-		tx.Rollback()
 		return err
 	}
 	defer stmt.Close()
 
 	for k, v := range config {
+		if k == "raw.lxc" {
+			err := validateRawLxc(config["raw.lxc"])
+			if err != nil {
+				return err
+			}
+		}
+
 		if !ValidContainerConfigKey(k) {
 			return fmt.Errorf("Bad key: %s\n", k)
 		}
@@ -567,7 +933,6 @@ func dbInsertProfiles(tx *sql.Tx, id int, profiles []string) error {
 		(?, (SELECT id FROM profiles WHERE name=?), ?);`
 	stmt, err := tx.Prepare(str)
 	if err != nil {
-		tx.Rollback()
 		return err
 	}
 	defer stmt.Close()
@@ -609,6 +974,8 @@ func ValidContainerConfigKey(k string) bool {
 		return true
 	case "raw.lxc":
 		return true
+	case "volatile.baseImage":
+		return true
 	}
 
 	if _, err := ExtractInterfaceFromConfigName(k); err == nil {
@@ -634,7 +1001,6 @@ func emptyProfile(l []string) bool {
  */
 func containerPut(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
-	shared.Debugf("containerPut: called with name %s\n", name)
 	cId, err := dbGetContainerId(d.db, name)
 	if err != nil {
 		return NotFound
@@ -647,7 +1013,7 @@ func containerPut(d *Daemon, r *http.Request) Response {
 
 	do := func() error {
 
-		tx, err := d.db.Begin()
+		tx, err := shared.DbBegin(d.db)
 		if err != nil {
 			return err
 		}
@@ -726,7 +1092,17 @@ func containerPost(d *Daemon, r *http.Request) Response {
 			return BadRequest(fmt.Errorf("renaming of running container not allowed"))
 		}
 
-		_, err := dbCreateContainer(d, body.Name, cTypeRegular, c.config, c.profiles, c.ephemeral)
+		args := DbCreateContainerArgs{
+			d:         d,
+			name:      body.Name,
+			ctype:     cTypeRegular,
+			config:    c.config,
+			profiles:  c.profiles,
+			ephem:     c.ephemeral,
+			baseImage: c.config["volatile.baseImage"],
+		}
+
+		_, err := dbCreateContainer(args)
 		if err != nil {
 			return SmartError(err)
 		}
@@ -753,19 +1129,12 @@ func containerDelete(d *Daemon, r *http.Request) Response {
 	if err != nil {
 		return SmartError(err)
 	}
-	dirsToDelete := containerDeleteSnapshots(d, name)
-	dbRemoveContainer(d, name)
-	dirsToDelete = append(dirsToDelete, shared.VarPath("lxc", name))
-	rmdir := func() error {
-		for _, dir := range dirsToDelete {
-			err := os.RemoveAll(dir)
-			if err != nil {
-				shared.Debugf("Error cleaning up %s: %s\n", dir, err)
-			}
-		}
-		return nil
+
+	rmct := func() error {
+		return removeContainer(d, name)
 	}
-	return AsyncResponse(shared.OperationWrap(rmdir), nil)
+
+	return AsyncResponse(shared.OperationWrap(rmct), nil)
 }
 
 var containerCmd = Command{name: "containers/{name}", get: containerGet, put: containerPut, delete: containerDelete, post: containerPost}
@@ -777,7 +1146,12 @@ func containerStateGet(d *Daemon, r *http.Request) Response {
 		return SmartError(err)
 	}
 
-	return SyncResponse(true, c.RenderState().Status)
+	state, err := c.RenderState()
+	if err != nil {
+		return InternalError(err)
+	}
+
+	return SyncResponse(true, state.Status)
 }
 
 type containerStatePutReq struct {
@@ -797,39 +1171,50 @@ type lxdContainer struct {
 	ephemeral bool
 }
 
-func (c *lxdContainer) RenderState() *shared.ContainerState {
-	return &shared.ContainerState{
-		Name:      c.name,
-		Profiles:  c.profiles,
-		Config:    c.config,
-		Userdata:  []byte{},
-		Status:    shared.NewStatus(c.c, c.c.State()),
-		Devices:   c.devices,
-		Ephemeral: c.ephemeral,
+func (c *lxdContainer) RenderState() (*shared.ContainerState, error) {
+	devices, err := dbGetDevices(c.daemon, c.name, false)
+	if err != nil {
+		return nil, err
 	}
+
+	config, err := dbGetConfig(c.daemon, c)
+	if err != nil {
+		return nil, err
+	}
+
+	return &shared.ContainerState{
+		Name:            c.name,
+		Profiles:        c.profiles,
+		Config:          config,
+		ExpandedConfig:  c.config,
+		Userdata:        []byte{},
+		Status:          shared.NewStatus(c.c, c.c.State()),
+		Devices:         devices,
+		ExpandedDevices: c.devices,
+		Ephemeral:       c.ephemeral,
+	}, nil
 }
 
 func (c *lxdContainer) Start() error {
-	err := c.c.Start()
+
+	f, err := ioutil.TempFile("", "lxd_lxc_startconfig_")
+	if err != nil {
+		return err
+	}
+	configPath := f.Name()
+	if err = f.Chmod(0600); err != nil {
+		f.Close()
+		os.Remove(configPath)
+		return err
+	}
+	f.Close()
+	err = c.c.SaveConfigFile(configPath)
+
+	cmd := exec.Command(os.Args[0], "forkstart", c.name, c.daemon.lxcpath, configPath)
+	err = cmd.Run()
 
 	if err == nil && c.ephemeral == true {
-		go func() {
-			c.c.Wait(lxc.STOPPED, -1*time.Second)
-			c.c.Wait(lxc.RUNNING, 1*time.Second)
-			c.c.Wait(lxc.STOPPED, -1*time.Second)
-
-			_, err := dbGetContainerId(c.daemon.db, c.name)
-			if err != nil {
-				return
-			}
-
-			dirsToDelete := containerDeleteSnapshots(c.daemon, c.name)
-			dbRemoveContainer(c.daemon, c.name)
-			dirsToDelete = append(dirsToDelete, shared.VarPath("lxc", c.name))
-			for _, dir := range dirsToDelete {
-				os.RemoveAll(dir)
-			}
-		}()
+		containerWatchEphemeral(c)
 	}
 
 	return err
@@ -865,7 +1250,18 @@ func (c *lxdContainer) Unfreeze() error {
 	return c.c.Unfreeze()
 }
 
-func (d *lxdContainer) applyConfig(config map[string]string) error {
+func validateRawLxc(rawLxc string) error {
+	for _, line := range strings.Split(rawLxc, "\n") {
+		membs := strings.SplitN(line, "=", 2)
+		if strings.ToLower(strings.Trim(membs[0], " \t")) == "lxc.logfile" {
+			return fmt.Errorf("setting lxc.logfile is not allowed")
+		}
+	}
+
+	return nil
+}
+
+func (d *lxdContainer) applyConfig(config map[string]string, fromProfile bool) error {
 	var err error
 	for k, v := range config {
 		switch k {
@@ -900,7 +1296,15 @@ func (d *lxdContainer) applyConfig(config map[string]string) error {
 		}
 	}
 
+	if fromProfile {
+		return nil
+	}
+
 	if lxcConfig, ok := config["raw.lxc"]; ok {
+		if err := validateRawLxc(lxcConfig); err != nil {
+			return err
+		}
+
 		f, err := ioutil.TempFile("", "lxd_config_")
 		if err != nil {
 			return err
@@ -922,33 +1326,33 @@ func (d *lxdContainer) applyConfig(config map[string]string) error {
 }
 
 func applyProfile(daemon *Daemon, d *lxdContainer, p string) error {
-	rows, err := shared.DbQuery(daemon.db, `SELECT key, value FROM profiles_config
+	q := `SELECT key, value FROM profiles_config
 		JOIN profiles ON profiles.id=profiles_config.profile_id
-		WHERE profiles.name=?`, p)
+		WHERE profiles.name=?`
+	var k, v string
+	inargs := []interface{}{p}
+	outfmt := []interface{}{k, v}
+	result, err := shared.DbQueryScan(daemon.db, q, inargs, outfmt)
 
 	if err != nil {
 		return err
 	}
-	defer rows.Close()
 
 	config := map[string]string{}
+	for _, r := range result {
+		k = r[0].(string)
+		v = r[1].(string)
 
-	for rows.Next() {
-		var k string
-		var v string
-		err = rows.Scan(&k, &v)
-		if err != nil {
-			fmt.Printf("DBERR: applyProfile: scan returned error %q\n", err)
-			return err
-		}
 		shared.Debugf("applying %s: %s", k, v)
+		if k == "raw.lxc" {
+			if _, ok := d.config["raw.lxc"]; ok {
+				shared.Debugf("ignoring overridden raw.lxc from profile")
+				continue
+			}
+		}
+
 		config[k] = v
 	}
-	err = rows.Err()
-	if err != nil {
-		fmt.Printf("DBERR: applyProfile: Err returned an error %q\n", err)
-		return err
-	}
 
 	newdevs, err := dbGetDevices(daemon, p, true)
 	if err != nil {
@@ -958,7 +1362,7 @@ func applyProfile(daemon *Daemon, d *lxdContainer, p string) error {
 		d.devices[k] = v
 	}
 
-	return d.applyConfig(config)
+	return d.applyConfig(config, true)
 }
 
 // GenerateMacAddr generates a mac address from a string template:
@@ -981,6 +1385,23 @@ func GenerateMacAddr(template string) (string, error) {
 	return ret.String(), nil
 }
 
+func (c *lxdContainer) updateContainerHWAddr(k, v string) {
+	for name, d := range c.devices {
+		if d["type"] != "nic" {
+			continue
+		}
+
+		for key, _ := range c.config {
+			device, err := ExtractInterfaceFromConfigName(key)
+			if err == nil && device == name {
+				d["hwaddr"] = v
+				c.config[key] = v
+				return
+			}
+		}
+	}
+}
+
 func (c *lxdContainer) setupMacAddresses(d *Daemon) error {
 	newConfigEntries := map[string]string{}
 
@@ -1025,17 +1446,80 @@ func (c *lxdContainer) setupMacAddresses(d *Daemon) error {
 
 	if len(newConfigEntries) > 0 {
 
-		tx, err := d.db.Begin()
+		tx, err := shared.DbBegin(d.db)
 		if err != nil {
 			return err
 		}
 
-		if err := dbInsertContainerConfig(tx, c.id, newConfigEntries); err != nil {
+		/*
+		 * My logic may be flawed here, but it seems to me that one of
+		 * the following must be true:
+		 * 1. The current database entry equals what we had stored.
+		 *    Our update akes precedence
+		 * 2. The current database entry is different from what we had
+		 *    stored.  Someone updated it since we last grabbed the
+		 *    container configuration.  So either
+		 *    a. it contains 'x' and is a template.  We have generated
+		 *       a real mac, so our update takes precedence
+		 *    b. it contains no 'x' and is an hwaddr, not template.  We
+		 *       defer to the racer's update since it may be actually
+		 *       starting the container.
+		 */
+		str := "INSERT INTO containers_config (container_id, key, value) values (?, ?, ?)"
+		stmt, err := tx.Prepare(str)
+		if err != nil {
 			tx.Rollback()
 			return err
 		}
+		defer stmt.Close()
 
-		return shared.TxCommit(tx)
+		ustr := "UPDATE containers_config SET value=? WHERE container_id=? AND key=?"
+		ustmt, err := tx.Prepare(ustr)
+		if err != nil {
+			tx.Rollback()
+			return err
+		}
+		defer ustmt.Close()
+
+		qstr := "SELECT value FROM containers_config WHERE container_id=? AND key=?"
+		qstmt, err := tx.Prepare(qstr)
+		if err != nil {
+			tx.Rollback()
+			return err
+		}
+		defer qstmt.Close()
+
+		for k, v := range newConfigEntries {
+			var racer string
+			err := qstmt.QueryRow(c.id, k).Scan(&racer)
+			if err == sql.ErrNoRows {
+				_, err = stmt.Exec(c.id, k, v)
+				if err != nil {
+					shared.Debugf("Error adding mac address to container\n")
+					tx.Rollback()
+					return err
+				}
+			} else if err != nil {
+				tx.Rollback()
+				return err
+			} else if strings.Contains(racer, "x") {
+				_, err = ustmt.Exec(v, c.id, k)
+				if err != nil {
+					shared.Debugf("Error updating mac address to container\n")
+					tx.Rollback()
+					return err
+				}
+			} else {
+				// we accept the racing task's update
+				c.updateContainerHWAddr(k, v)
+			}
+		}
+
+		err = shared.TxCommit(tx)
+		if err != nil {
+			fmt.Printf("setupMacAddresses: (TxCommit) error %s\n", err)
+		}
+		return err
 	}
 
 	return nil
@@ -1057,7 +1541,6 @@ func (c *lxdContainer) applyDevices() error {
 				return fmt.Errorf("Failed configuring device %s: %s\n", name, err)
 			}
 		}
-		shared.Debugf("Configured device %s\n", name)
 	}
 	return nil
 }
@@ -1197,7 +1680,7 @@ func newLxdContainer(name string, daemon *Daemon) (*lxdContainer, error) {
 		}
 	}
 
-	err = d.applyConfig(d.config)
+	err = d.applyConfig(d.config, false)
 	if err != nil {
 		return nil, err
 	}
@@ -1283,7 +1766,7 @@ func containerFileHandler(d *Daemon, r *http.Request) Response {
 	case "GET":
 		return containerFileGet(r, p)
 	case "POST":
-		return containerFilePut(r, p)
+		return containerFilePut(r, p, d.idMap, c.isPrivileged())
 	default:
 		return NotFound
 	}
@@ -1310,13 +1793,20 @@ func containerFileGet(r *http.Request, path string) Response {
 	return FileResponse(r, path, filepath.Base(path), headers)
 }
 
-func containerFilePut(r *http.Request, p string) Response {
+func containerFilePut(r *http.Request, p string, idmap *shared.Idmap, privileged bool) Response {
 
 	uid, gid, mode, err := shared.ParseLXDFileHeaders(r.Header)
 	if err != nil {
 		return BadRequest(err)
 	}
 
+	if !privileged {
+
+		// map provided uid / gid to UID / GID range of the container
+		uid = int(idmap.Uidmin) + uid
+		gid = int(idmap.Gidmin) + gid
+	}
+
 	fileinfo, err := os.Stat(path.Dir(p))
 	if err != nil {
 		return SmartError(err)
@@ -1374,30 +1864,23 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 
 	regexp := fmt.Sprintf("%s/", cname)
 	length := len(regexp)
-	rows, err := shared.DbQuery(d.db, "SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?",
-		cTypeSnapshot, length, regexp)
+	q := "SELECT name FROM containers WHERE type=? AND SUBSTR(name,1,?)=?"
+	var name string
+	inargs := []interface{}{cTypeSnapshot, length, regexp}
+	outfmt := []interface{}{name}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return SmartError(err)
 	}
-	defer rows.Close()
 
 	var body []string
 
-	for rows.Next() {
-		var name string
-		err = rows.Scan(&name)
-		if err != nil {
-			fmt.Printf("DBERR: containerSnapshotsGet: scan returned error %q\n", err)
-			return InternalError(err)
-		}
+	for _, r := range results {
+		name = r[0].(string)
+
 		url := fmt.Sprintf("/%s/containers/%s/snapshots/%s", shared.APIVersion, cname, name)
 		body = append(body, url)
 	}
-	err = rows.Err()
-	if err != nil {
-		fmt.Printf("DBERR: containerSnapshotsGet: Err returned an error %q\n", err)
-		return InternalError(err)
-	}
 
 	return SyncResponse(true, body)
 }
@@ -1410,24 +1893,23 @@ func nextSnapshot(d *Daemon, name string) int {
 	base := fmt.Sprintf("%s/snap", name)
 	length := len(base)
 	q := fmt.Sprintf("SELECT MAX(name) FROM containers WHERE type=? AND SUBSTR(name,1,?)=?")
-	rows, err := shared.DbQuery(d.db, q, cTypeSnapshot, length, base)
+	var numstr string
+	inargs := []interface{}{cTypeSnapshot, length, base}
+	outfmt := []interface{}{numstr}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return 0
 	}
-	defer rows.Close()
 	max := 0
-	for rows.Next() {
-		var tmp string
-		var num int
-		err = rows.Scan(&tmp)
-		if err != nil {
-			fmt.Printf("DBERR: nextSnapshot: scan returned error %q\n", err)
-		}
-		if len(tmp) <= length {
+
+	for _, r := range results {
+		numstr = r[0].(string)
+		if len(numstr) <= length {
 			continue
 		}
-		tmp2 := tmp[length:]
-		count, err := fmt.Sscanf(tmp2, "%d", &num)
+		substr := numstr[length:]
+		var num int
+		count, err := fmt.Sscanf(substr, "%d", &num)
 		if err != nil || count != 1 {
 			continue
 		}
@@ -1435,10 +1917,7 @@ func nextSnapshot(d *Daemon, name string) int {
 			max = num + 1
 		}
 	}
-	err = rows.Err()
-	if err != nil {
-		fmt.Printf("DBERR: nextSnapshot: Err returned an error %q\n", err)
-	}
+
 	return max
 }
 
@@ -1504,8 +1983,20 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 		}
 
 		/* Create the db info */
-		//cId, err := dbCreateContainer(d, snapshotName, cTypeSnapshot)
-		_, err := dbCreateContainer(d, fullName, cTypeSnapshot, c.config, c.profiles, c.ephemeral)
+		args := DbCreateContainerArgs{
+			d:         d,
+			name:      fullName,
+			ctype:     cTypeSnapshot,
+			config:    c.config,
+			profiles:  c.profiles,
+			ephem:     c.ephemeral,
+			baseImage: c.config["volatile.baseImage"],
+		}
+
+		_, err := dbCreateContainer(args)
+		if err != nil {
+			return err
+		}
 
 		/* Create the directory and rootfs, set perms */
 		/* Copy the rootfs */
@@ -1601,6 +2092,8 @@ var containerSnapshotCmd = Command{name: "containers/{name}/snapshots/{snapshotN
 type execWs struct {
 	command      []string
 	container    *lxc.Container
+	rootUid      int
+	rootGid      int
 	options      lxc.AttachOptions
 	conns        []*websocket.Conn
 	allConnected chan bool
@@ -1612,7 +2105,7 @@ type execWs struct {
 func (s *execWs) Metadata() interface{} {
 	fds := shared.Jmap{}
 	for fd, secret := range s.fds {
-		fds[string(fd)] = secret
+		fds[strconv.Itoa(fd)] = secret
 	}
 
 	return shared.Jmap{"fds": fds}
@@ -1667,7 +2160,7 @@ func (s *execWs) Do() shared.OperationResult {
 	if s.interactive {
 		ttys = make([]*os.File, 1)
 		ptys = make([]*os.File, 1)
-		ptys[0], ttys[0], err = shared.OpenPty()
+		ptys[0], ttys[0], err = shared.OpenPty(s.rootUid, s.rootGid)
 		s.options.StdinFd = ttys[0].Fd()
 		s.options.StdoutFd = ttys[0].Fd()
 		s.options.StderrFd = ttys[0].Fd()
@@ -1766,6 +2259,10 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 	if post.WaitForWS {
 		ws := &execWs{}
 		ws.fds = map[int]string{}
+		if !c.isPrivileged() {
+			ws.rootUid = int(d.idMap.Uidmin)
+			ws.rootGid = int(d.idMap.Gidmin)
+		}
 		if post.Interactive {
 			ws.conns = make([]*websocket.Conn, 1)
 		} else {
@@ -1808,3 +2305,29 @@ func containerExecPost(d *Daemon, r *http.Request) Response {
 }
 
 var containerExecCmd = Command{name: "containers/{name}/exec", post: containerExecPost}
+
+/*
+ * This is called by lxd when called as "lxd forkstart <container>"
+ * 'forkstart' is used instead of just 'start' in the hopes that people
+ * do not accidentally type 'lxd start' instead of 'lxc start'
+ *
+ * We expect to read the lxcconfig over fd 3.
+ */
+func startContainer(args []string) error {
+	if len(args) != 4 {
+		return fmt.Errorf("Bad arguments: %q\n", args)
+	}
+	name := args[1]
+	lxcpath := args[2]
+	configPath := args[3]
+	c, err := lxc.NewContainer(name, lxcpath)
+	if err != nil {
+		return fmt.Errorf("Error initializing container for start: %q", err)
+	}
+	err = c.LoadConfigFile(configPath)
+	if err != nil {
+		return fmt.Errorf("Error opening startup config file: %q", err)
+	}
+	os.Remove(configPath)
+	return c.Start()
+}
