@@ -12,19 +12,10 @@ import (
 	_ "github.com/mattn/go-sqlite3"
 )
 
-const DB_CURRENT_VERSION int = 6
+const DB_CURRENT_VERSION int = 7
 
-var (
-	DbErrAlreadyDefined = fmt.Errorf("already exists")
-)
-
-func createDb(p string) (*sql.DB, error) {
-	db, err := sql.Open("sqlite3", p)
-	if err != nil {
-		return nil, fmt.Errorf("Error creating database: %s\n", err)
-	}
-	stmt := `
-CREATE TABLE certificates (
+const CURRENT_SCHEMA string = `
+CREATE TABLE IF NOT EXISTS certificates (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     fingerprint VARCHAR(255) NOT NULL,
     type INTEGER NOT NULL,
@@ -32,13 +23,13 @@ CREATE TABLE certificates (
     certificate TEXT NOT NULL,
     UNIQUE (fingerprint)
 );
-CREATE TABLE config (
+CREATE TABLE IF NOT EXISTS config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     key VARCHAR(255) NOT NULL,
     value TEXT,
     UNIQUE (key)
 );
-CREATE TABLE containers (
+CREATE TABLE IF NOT EXISTS containers (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
     architecture INTEGER NOT NULL,
@@ -47,40 +38,40 @@ CREATE TABLE containers (
     ephemeral INTEGER NOT NULL DEFAULT 0,
     UNIQUE (name)
 );
-CREATE TABLE containers_config (
+CREATE TABLE IF NOT EXISTS containers_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     container_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
     value TEXT,
-    FOREIGN KEY (container_id) REFERENCES containers (id),
+    FOREIGN KEY (container_id) REFERENCES containers (id) ON DELETE CASCADE,
     UNIQUE (container_id, key)
 );
-CREATE TABLE containers_devices (
+CREATE TABLE IF NOT EXISTS containers_devices (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     container_id INTEGER NOT NULL,
     name VARCHAR(255) NOT NULL,
     type INTEGER NOT NULL default 0,
-    FOREIGN KEY (container_id) REFERENCES containers (id),
+    FOREIGN KEY (container_id) REFERENCES containers (id) ON DELETE CASCADE,
     UNIQUE (container_id, name)
 );
-CREATE TABLE containers_devices_config (
+CREATE TABLE IF NOT EXISTS containers_devices_config (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     container_device_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
     value TEXT,
-    FOREIGN KEY (container_device_id) REFERENCES containers_devices (id),
+    FOREIGN KEY (container_device_id) REFERENCES containers_devices (id) ON DELETE CASCADE,
     UNIQUE (container_device_id, key)
 );
-CREATE TABLE containers_profiles (
+CREATE TABLE IF NOT EXISTS containers_profiles (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     container_id INTEGER NOT NULL,
     profile_id INTEGER NOT NULL,
     apply_order INTEGER NOT NULL default 0,
     UNIQUE (container_id, profile_id),
-    FOREIGN KEY (container_id) REFERENCES containers(id),
-    FOREIGN KEY (profile_id) REFERENCES profiles(id)
+    FOREIGN KEY (container_id) REFERENCES containers(id) ON DELETE CASCADE,
+    FOREIGN KEY (profile_id) REFERENCES profiles(id) ON DELETE CASCADE
 );
-CREATE TABLE images (
+CREATE TABLE IF NOT EXISTS images (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     fingerprint VARCHAR(255) NOT NULL,
     filename VARCHAR(255) NOT NULL,
@@ -92,81 +83,260 @@ CREATE TABLE images (
     upload_date DATETIME NOT NULL,
     UNIQUE (fingerprint)
 );
-CREATE TABLE images_aliases (
+CREATE TABLE IF NOT EXISTS images_aliases (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
     image_id INTEGER NOT NULL,
     description VARCHAR(255),
-    FOREIGN KEY (image_id) REFERENCES images (id),
+    FOREIGN KEY (image_id) REFERENCES images (id) ON DELETE CASCADE,
     UNIQUE (name)
 );
-CREATE TABLE images_properties (
+CREATE TABLE IF NOT EXISTS images_properties (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     image_id INTEGER NOT NULL,
     type INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
     value TEXT,
-    FOREIGN KEY (image_id) REFERENCES images (id)
+    FOREIGN KEY (image_id) REFERENCES images (id) ON DELETE CASCADE
 );
-CREATE TABLE profiles (
+CREATE TABLE IF NOT EXISTS profiles (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
     UNIQUE (name)
 );
-CREATE TABLE profiles_config (
+CREATE TABLE IF NOT EXISTS profiles_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     profile_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
     value VARCHAR(255),
     UNIQUE (profile_id, key),
-    FOREIGN KEY (profile_id) REFERENCES profiles(id)
+    FOREIGN KEY (profile_id) REFERENCES profiles(id) ON DELETE CASCADE
 );
-CREATE TABLE profiles_devices (
+CREATE TABLE IF NOT EXISTS profiles_devices (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     profile_id INTEGER NOT NULL,
     name VARCHAR(255) NOT NULL,
     type INTEGER NOT NULL default 0,
     UNIQUE (profile_id, name),
-    FOREIGN KEY (profile_id) REFERENCES profiles (id)
+    FOREIGN KEY (profile_id) REFERENCES profiles (id) ON DELETE CASCADE
 );
-CREATE TABLE profiles_devices_config (
+CREATE TABLE IF NOT EXISTS profiles_devices_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     profile_device_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
     value TEXT,
     UNIQUE (profile_device_id, key),
-    FOREIGN KEY (profile_device_id) REFERENCES profiles_devices (id)
+    FOREIGN KEY (profile_device_id) REFERENCES profiles_devices (id) ON DELETE CASCADE
 );
-CREATE TABLE schema (
+CREATE TABLE IF NOT EXISTS schema (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     version INTEGER NOT NULL,
     updated_at DATETIME NOT NULL,
     UNIQUE (version)
-);
-INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
+);`
 
-	_, err = db.Exec(stmt, DB_CURRENT_VERSION)
+var (
+	DbErrAlreadyDefined = fmt.Errorf("already exists")
+)
+
+// Create the initial (current) schema for a given SQLite DB connection.
+// This should stay indempotent.
+func createDb(db *sql.DB) (err error) {
+	_, err = db.Exec(CURRENT_SCHEMA)
 	if err != nil {
-		return db, err
+		return err
+	}
+
+	// To make the schema creation indempotent, only insert the schema version
+	// if there isn't one already.
+	latest_version := getSchema(db)
+
+	if latest_version == 0 {
+		// There isn't an entry for schema version, let's put it in.
+		insert_stmt := `INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
+		_, err = db.Exec(insert_stmt, DB_CURRENT_VERSION)
+		if err != nil {
+			return err
+		}
 	}
 
 	err = createDefaultProfile(db)
 
-	return db, err
+	return err
 }
 
-func getSchema(db *sql.DB) (int, error) {
-	rows, err := db.Query("SELECT max(version) FROM schema")
+func getSchema(db *sql.DB) (v int) {
+	arg1 := []interface{}{}
+	arg2 := []interface{}{&v}
+	q := "SELECT max(version) FROM schema"
+	err := shared.DbQueryRowScan(db, q, arg1, arg2)
 	if err != nil {
-		return 0, nil
+		return 0
 	}
+	return v
+}
+
+func updateFromV6(db *sql.DB) error {
+	// This update recreates the schemas that need an ON DELETE CASCADE foreign
+	// key.
+	stmt := `
+PRAGMA foreign_keys=OFF; -- So that integrity doesn't get in the way for now
+
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_config; 
+DROP TABLE containers_config;
+CREATE TABLE IF NOT EXISTS containers_config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    container_id INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    FOREIGN KEY (container_id) REFERENCES containers (id) ON DELETE CASCADE,
+    UNIQUE (container_id, key)
+);
+INSERT INTO containers_config SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_devices; 
+DROP TABLE containers_devices;
+CREATE TABLE IF NOT EXISTS containers_devices (
+    id INTEGER primary key AUTOINCREMENT NOT NULL,
+    container_id INTEGER NOT NULL,
+    name VARCHAR(255) NOT NULL,
+    type INTEGER NOT NULL default 0,
+    FOREIGN KEY (container_id) REFERENCES containers (id) ON DELETE CASCADE,
+    UNIQUE (container_id, name)
+);
+INSERT INTO containers_devices SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_devices_config; 
+DROP TABLE containers_devices_config;
+CREATE TABLE IF NOT EXISTS containers_devices_config (
+    id INTEGER primary key AUTOINCREMENT NOT NULL,
+    container_device_id INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    FOREIGN KEY (container_device_id) REFERENCES containers_devices (id) ON DELETE CASCADE,
+    UNIQUE (container_device_id, key)
+);
+INSERT INTO containers_devices_config SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM containers_profiles; 
+DROP TABLE containers_profiles;
+CREATE TABLE IF NOT EXISTS containers_profiles (
+    id INTEGER primary key AUTOINCREMENT NOT NULL,
+    container_id INTEGER NOT NULL,
+    profile_id INTEGER NOT NULL,
+    apply_order INTEGER NOT NULL default 0,
+    UNIQUE (container_id, profile_id),
+    FOREIGN KEY (container_id) REFERENCES containers(id) ON DELETE CASCADE,
+    FOREIGN KEY (profile_id) REFERENCES profiles(id) ON DELETE CASCADE
+);
+INSERT INTO containers_profiles SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM images_aliases; 
+DROP TABLE images_aliases;
+CREATE TABLE IF NOT EXISTS images_aliases (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    name VARCHAR(255) NOT NULL,
+    image_id INTEGER NOT NULL,
+    description VARCHAR(255),
+    FOREIGN KEY (image_id) REFERENCES images (id) ON DELETE CASCADE,
+    UNIQUE (name)
+);
+INSERT INTO images_aliases SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM images_properties; 
+DROP TABLE images_properties;
+CREATE TABLE IF NOT EXISTS images_properties (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    image_id INTEGER NOT NULL,
+    type INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    FOREIGN KEY (image_id) REFERENCES images (id) ON DELETE CASCADE
+);
+INSERT INTO images_properties SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM profiles_config; 
+DROP TABLE profiles_config;
+CREATE TABLE IF NOT EXISTS profiles_config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    profile_id INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value VARCHAR(255),
+    UNIQUE (profile_id, key),
+    FOREIGN KEY (profile_id) REFERENCES profiles(id) ON DELETE CASCADE
+);
+INSERT INTO profiles_config SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM profiles_devices; 
+DROP TABLE profiles_devices;
+CREATE TABLE IF NOT EXISTS profiles_devices (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    profile_id INTEGER NOT NULL,
+    name VARCHAR(255) NOT NULL,
+    type INTEGER NOT NULL default 0,
+    UNIQUE (profile_id, name),
+    FOREIGN KEY (profile_id) REFERENCES profiles (id) ON DELETE CASCADE
+);
+INSERT INTO profiles_devices SELECT * FROM tmp;
+DROP TABLE tmp;
+
+CREATE TEMP TABLE tmp AS SELECT * FROM profiles_devices_config; 
+DROP TABLE profiles_devices_config;
+CREATE TABLE IF NOT EXISTS profiles_devices_config (
+    id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+    profile_device_id INTEGER NOT NULL,
+    key VARCHAR(255) NOT NULL,
+    value TEXT,
+    UNIQUE (profile_device_id, key),
+    FOREIGN KEY (profile_device_id) REFERENCES profiles_devices (id) ON DELETE CASCADE
+);
+INSERT INTO profiles_devices_config SELECT * FROM tmp;
+DROP TABLE tmp;
+
+PRAGMA foreign_keys=ON; -- Make sure we turn integrity checks back on.
+INSERT INTO schema (version, updated_at) VALUES (?, strftime("%s"));`
+	_, err := db.Exec(stmt, 7)
+	if err != nil {
+		return err
+	}
+
+	// Get the rows with broken foreign keys an nuke them
+	rows, err := db.Query("PRAGMA foreign_key_check;")
+	if err != nil {
+		return err
+	}
+
+	var tablestodelete []string
+	var rowidtodelete []int
+
 	defer rows.Close()
 	for rows.Next() {
-		var v int
-		rows.Scan(&v)
-		return v, nil
+		var tablename string
+		var rowid int
+		var targetname string
+		var keynumber int
+
+		rows.Scan(&tablename, &rowid, &targetname, &keynumber)
+		tablestodelete = append(tablestodelete, tablename)
+		rowidtodelete = append(rowidtodelete, rowid)
 	}
-	return 0, nil
+	rows.Close()
+
+	for i := range tablestodelete {
+		_, err = db.Exec(fmt.Sprintf("DELETE FROM %s WHERE rowid = %d;", tablestodelete[i], rowidtodelete[i]))
+		if err != nil {
+			return err
+		}
+	}
+
+	return err
 }
 
 func updateFromV5(db *sql.DB) error {
@@ -180,7 +350,7 @@ INSERT INTO schema (version, updated_at) VALUES (?, strftime("%s"));`
 
 func updateFromV4(db *sql.DB) error {
 	stmt := `
-CREATE TABLE config (
+CREATE TABLE IF NOT EXISTS config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     key VARCHAR(255) NOT NULL,
     value TEXT,
@@ -228,15 +398,15 @@ func updateFromV3(db *sql.DB) error {
 
 func updateFromV2(db *sql.DB) error {
 	stmt := `
-CREATE TABLE containers_devices (
+CREATE TABLE IF NOT EXISTS containers_devices (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     container_id INTEGER NOT NULL,
     name VARCHAR(255) NOT NULL,
     type INTEGER NOT NULL default 0,
-    FOREIGN KEY (container_id) REFERENCES containers (id),
+    FOREIGN KEY (container_id) REFERENCES containers (id) ON DELETE CASCADE,
     UNIQUE (container_id, name)
 );
-CREATE TABLE containers_devices_config (
+CREATE TABLE IF NOT EXISTS containers_devices_config (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     container_device_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
@@ -244,37 +414,37 @@ CREATE TABLE containers_devices_config (
     FOREIGN KEY (container_device_id) REFERENCES containers_devices (id),
     UNIQUE (container_device_id, key)
 );
-CREATE TABLE containers_profiles (
+CREATE TABLE IF NOT EXISTS containers_profiles (
     id INTEGER primary key AUTOINCREMENT NOT NULL,
     container_id INTEGER NOT NULL,
     profile_id INTEGER NOT NULL,
     apply_order INTEGER NOT NULL default 0,
     UNIQUE (container_id, profile_id),
-    FOREIGN KEY (container_id) REFERENCES containers(id),
-    FOREIGN KEY (profile_id) REFERENCES profiles(id)
+    FOREIGN KEY (container_id) REFERENCES containers(id) ON DELETE CASCADE,
+    FOREIGN KEY (profile_id) REFERENCES profiles(id) ON DELETE CASCADE
 );
-CREATE TABLE profiles (
+CREATE TABLE IF NOT EXISTS profiles (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
     UNIQUE (name)
 );
-CREATE TABLE profiles_config (
+CREATE TABLE IF NOT EXISTS profiles_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     profile_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
     value VARCHAR(255),
     UNIQUE (profile_id, key),
-    FOREIGN KEY (profile_id) REFERENCES profiles(id)
+    FOREIGN KEY (profile_id) REFERENCES profiles(id) ON DELETE CASCADE
 );
-CREATE TABLE profiles_devices (
+CREATE TABLE IF NOT EXISTS profiles_devices (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     profile_id INTEGER NOT NULL,
     name VARCHAR(255) NOT NULL,
     type INTEGER NOT NULL default 0,
     UNIQUE (profile_id, name),
-    FOREIGN KEY (profile_id) REFERENCES profiles (id)
+    FOREIGN KEY (profile_id) REFERENCES profiles (id) ON DELETE CASCADE
 );
-CREATE TABLE profiles_devices_config (
+CREATE TABLE IF NOT EXISTS profiles_devices_config (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     profile_device_id INTEGER NOT NULL,
     key VARCHAR(255) NOT NULL,
@@ -291,12 +461,12 @@ INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
 func updateFromV1(db *sql.DB) error {
 	// v1..v2 adds images aliases
 	stmt := `
-CREATE TABLE images_aliases (
+CREATE TABLE IF NOT EXISTS images_aliases (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     name VARCHAR(255) NOT NULL,
     image_id INTEGER NOT NULL,
     description VARCHAR(255),
-    FOREIGN KEY (image_id) REFERENCES images (id),
+    FOREIGN KEY (image_id) REFERENCES images (id) ON DELETE CASCADE,
     UNIQUE (name)
 );
 INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
@@ -307,7 +477,7 @@ INSERT INTO schema (version, updated_at) values (?, strftime("%s"));`
 func updateFromV0(db *sql.DB) error {
 	// v0..v1 adds schema table
 	stmt := `
-CREATE TABLE schema (
+CREATE TABLE IF NOT EXISTS schema (
     id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
     version INTEGER NOT NULL,
     updated_at DATETIME NOT NULL,
@@ -362,6 +532,13 @@ func updateDb(db *sql.DB, prev_version int) error {
 			return err
 		}
 	}
+	if prev_version < 7 {
+		err = updateFromV6(db)
+		if err != nil {
+			return err
+		}
+	}
+
 	return nil
 }
 
@@ -382,7 +559,7 @@ func createDefaultProfile(db *sql.DB) error {
 		return nil
 	}
 
-	tx, err := db.Begin()
+	tx, err := shared.DbBegin(db)
 	if err != nil {
 		return err
 	}
@@ -436,39 +613,48 @@ func createDefaultProfile(db *sql.DB) error {
 	return nil
 }
 
-func initDb(d *Daemon) error {
-	dbpath := shared.VarPath("lxd.db")
-	var db *sql.DB
-	var err error
-	timeout := 30 // TODO - make this command-line configurable?
-	openPath := fmt.Sprintf("%s?_busy_timeout=%d&_txlock=immediate", dbpath, timeout*1000)
-	if !shared.PathExists(dbpath) {
-		db, err = createDb(openPath)
-		if err != nil {
-			return err
-		}
-	} else {
-		db, err = sql.Open("sqlite3", openPath)
-		if err != nil {
-			return err
-		}
-	}
+// Create a database connection object and return it.
+func initializeDbObject(path string) (db *sql.DB, err error) {
+	var openPath string
 
-	d.db = db
+	timeout := 5 // TODO - make this command-line configurable?
 
-	v, err := getSchema(db)
+	// These are used to tune the transaction BEGIN behavior instead of using the
+	// similar "locking_mode" pragma (locking for the whole database connection).
+	openPath = fmt.Sprintf("%s?_busy_timeout=%d&_txlock=exclusive", path, timeout*1000)
+
+	// Open the database. If the file doesn't exist it is created.
+	db, err = sql.Open("sqlite3", openPath)
 	if err != nil {
-		return fmt.Errorf("Bad database, or database too new for this lxd version")
+		return nil, err
 	}
 
+	// Table creation is indempotent, run it every time
+	err = createDb(db)
+	if err != nil {
+		return nil, fmt.Errorf("Error creating database: %s\n", err)
+	}
+
+	// Run PRAGMA statements now since they are *per-connection*.
+	db.Exec("PRAGMA foreign_keys=ON;") // This allows us to use ON DELETE CASCADE
+
+	v := getSchema(db)
+
 	if v != DB_CURRENT_VERSION {
 		err = updateDb(db, v)
 		if err != nil {
-			return err
+			return nil, err
 		}
 	}
 
-	return nil
+	return db, nil
+}
+
+// Initialize a database connection and set it on the daemon.
+func initDb(d *Daemon) (err error) {
+	path := shared.VarPath("lxd.db")
+	d.db, err = initializeDbObject(path)
+	return err
 }
 
 var NoSuchImageError = errors.New("No such image")
@@ -578,19 +764,20 @@ func dbAddAlias(d *Daemon, name string, tgt int, desc string) error {
 
 func dbGetConfig(d *Daemon, c *lxdContainer) (map[string]string, error) {
 	q := `SELECT key, value FROM containers_config WHERE container_id=?`
-	rows, err := shared.DbQuery(d.db, q, c.id)
+	var key, value string
+	inargs := []interface{}{c.id}
+	outfmt := []interface{}{key, value}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
-	defer rows.Close()
 
 	config := map[string]string{}
 
-	for rows.Next() {
-		var key, value string
-		if err := rows.Scan(&key, &value); err != nil {
-			return nil, err
-		}
+	for _, r := range results {
+		key = r[0].(string)
+		value = r[1].(string)
+
 		config[key] = value
 	}
 
@@ -613,19 +800,20 @@ func dbGetProfileConfig(d *Daemon, name string) (map[string]string, error) {
 	q = `SELECT key, value FROM profiles_config JOIN profiles
 		ON profiles_config.profile_id=profiles.id
 		WHERE name=?`
-	rows, err := shared.DbQuery(d.db, q, name)
+	var key, value string
+	inargs := []interface{}{name}
+	outfmt := []interface{}{key, value}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
-	defer rows.Close()
 
 	config := map[string]string{}
 
-	for rows.Next() {
-		var key, value string
-		if err := rows.Scan(&key, &value); err != nil {
-			return nil, err
-		}
+	for _, r := range results {
+		key = r[0].(string)
+		value = r[1].(string)
+
 		config[key] = value
 	}
 
@@ -642,72 +830,82 @@ func dbGetProfiles(d *Daemon, c *lxdContainer) ([]string, error) {
 	q := `SELECT name FROM containers_profiles JOIN profiles
 		ON containers_profiles.profile_id=profiles.id
 		WHERE container_id=? ORDER BY containers_profiles.apply_order`
-	rows, err := shared.DbQuery(d.db, q, c.id)
+	var name string
+	inargs := []interface{}{c.id}
+	outfmt := []interface{}{name}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
-	defer rows.Close()
 
 	var profiles []string
 
-	for rows.Next() {
-		var name string
-		err := rows.Scan(&name)
-		if err != nil {
-			return nil, err
-		}
+	for _, r := range results {
+		name = r[0].(string)
+
 		profiles = append(profiles, name)
 	}
 
 	return profiles, nil
 }
 
+func dbGetDeviceConfig(db *sql.DB, id int, isprofile bool) (shared.Device, error) {
+	var q string
+	if isprofile {
+		q = `SELECT key, value FROM profiles_devices_config WHERE profile_device_id=?`
+	} else {
+		q = `SELECT key, value FROM containers_devices_config WHERE container_device_id=?`
+	}
+	newdev := shared.Device{}
+	var key, value string
+	inargs := []interface{}{id}
+	outfmt := []interface{}{key, value}
+	results, err := shared.DbQueryScan(db, q, inargs, outfmt)
+	if err != nil {
+		return newdev, err
+	}
+
+	for _, r := range results {
+		key = r[0].(string)
+		value = r[1].(string)
+		newdev[key] = value
+	}
+
+	return newdev, nil
+}
+
 func dbGetDevices(d *Daemon, qName string, isprofile bool) (shared.Devices, error) {
-	var q, q2 string
+	var q string
 	if isprofile {
 		q = `SELECT profiles_devices.id, profiles_devices.name, profiles_devices.type
 			FROM profiles_devices JOIN profiles
 			ON profiles_devices.profile_id = profiles.id
 			WHERE profiles.name=?`
-		q2 = `SELECT key, value FROM profiles_devices_config WHERE profile_device_id=?`
 	} else {
 		q = `SELECT containers_devices.id, containers_devices.name, containers_devices.type
 			FROM containers_devices JOIN containers
 			ON containers_devices.container_id = containers.id
 			WHERE containers.name=?`
-		q2 = `SELECT key, value FROM containers_devices_config WHERE container_device_id=?`
 	}
-	rows, err := shared.DbQuery(d.db, q, qName)
+	var id int
+	var name, dtype string
+	inargs := []interface{}{qName}
+	outfmt := []interface{}{id, name, dtype}
+	results, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
-	defer rows.Close()
 
 	devices := shared.Devices{}
-
-	for rows.Next() {
-		var id int
-		var name, dtype string
-		if err := rows.Scan(&id, &name, &dtype); err != nil {
-			return nil, err
-		}
-		newdev := shared.Device{}
-		rows2, err := shared.DbQuery(d.db, q2, id)
+	for _, r := range results {
+		id = r[0].(int)
+		name = r[1].(string)
+		dtype = r[2].(string)
+		newdev, err := dbGetDeviceConfig(d.db, id, isprofile)
 		if err != nil {
 			return nil, err
 		}
-		defer rows2.Close()
-
 		newdev["type"] = dtype
-		for rows2.Next() {
-			var k, v string
-			rows2.Scan(&k, &v)
-			if !shared.ValidDeviceConfig(dtype, k, v) {
-				return nil, fmt.Errorf("Invalid config for device type %s: %s = %s\n", dtype, k, v)
-			}
-			newdev[k] = v
-		}
-
 		devices[name] = newdev
 	}
 
