@@ -7,6 +7,7 @@ import (
 	"os"
 	"os/exec"
 	"os/signal"
+	"runtime/pprof"
 	"syscall"
 	"time"
 
@@ -27,6 +28,9 @@ var listenAddr = gnuflag.String("tcp", "", "TCP address <addr:port> to listen on
 var group = gnuflag.String("group", "", "Group which owns the shared socket")
 var help = gnuflag.Bool("help", false, "Print this help message.")
 var version = gnuflag.Bool("version", false, "Print LXD's version number and exit.")
+var printGoroutines = gnuflag.Int("print-goroutines-every", -1, "For debugging, print a complete stack trace every n seconds")
+var cpuProfile = gnuflag.String("cpuprofile", "", "Enable cpu profiling into the specified file.")
+var memProfile = gnuflag.String("memprofile", "", "Enable memory profiling into the specified file.")
 
 func init() {
 	myGroup, err := shared.GroupName(os.Getgid())
@@ -39,6 +43,10 @@ func init() {
 }
 
 func run() error {
+	if len(os.Args) > 1 && os.Args[1] == "forkstart" {
+		return startContainer(os.Args[1:])
+	}
+
 	gnuflag.Usage = func() {
 		fmt.Printf("Usage: lxd [options]\n\nOptions:\n")
 		gnuflag.PrintDefaults()
@@ -63,6 +71,25 @@ func run() error {
 		shared.SetDebug(*debug)
 	}
 
+	if gnuflag.NArg() != 0 {
+		gnuflag.Usage()
+		return fmt.Errorf("Unknown arguments")
+	}
+
+	if *cpuProfile != "" {
+		f, err := os.Create(*cpuProfile)
+		if err != nil {
+			fmt.Printf("Error opening cpu profile file: %s\n", err)
+			return nil
+		}
+		pprof.StartCPUProfile(f)
+		defer pprof.StopCPUProfile()
+	}
+
+	if *memProfile != "" {
+		go memProfiler()
+	}
+
 	needed_programs := []string{"setfacl", "rsync", "tar", "xz"}
 	for _, p := range needed_programs {
 		_, err := exec.LookPath(p)
@@ -71,6 +98,15 @@ func run() error {
 		}
 	}
 
+	if *printGoroutines > 0 {
+		go func() {
+			for {
+				time.Sleep(time.Duration(*printGoroutines) * time.Second)
+				shared.PrintStack()
+			}
+		}()
+	}
+
 	d, err := StartDaemon(*listenAddr)
 
 	if err != nil {
