@@ -38,30 +38,21 @@ type profilesPostReq struct {
 
 func profilesGet(d *Daemon, r *http.Request) Response {
 	q := fmt.Sprintf("SELECT name FROM profiles")
-	rows, err := shared.DbQuery(d.db, q)
+	inargs := []interface{}{}
+	var name string
+	outfmt := []interface{}{name}
+	result, err := shared.DbQueryScan(d.db, q, inargs, outfmt)
 	if err != nil {
 		return SmartError(err)
 	}
-	defer rows.Close()
-
-	result := []string{}
-	for rows.Next() {
-		name := ""
-		err = rows.Scan(&name)
-		if err != nil {
-			fmt.Printf("DBERR: profilesGet: scan returned error %q\n", err)
-			return InternalError(err)
-		}
-
-		result = append(result, fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, name))
-	}
-	err = rows.Err()
-	if err != nil {
-		fmt.Printf("DBERR: profilesGet: Err returned an error %q\n", err)
-		return InternalError(err)
+	response := []string{}
+	for _, r := range result {
+		name := r[0].(string)
+		url := fmt.Sprintf("/%s/profiles/%s", shared.APIVersion, name)
+		response = append(response, url)
 	}
 
-	return SyncResponse(true, result)
+	return SyncResponse(true, response)
 }
 
 func profilesPost(d *Daemon, r *http.Request) Response {
@@ -76,7 +67,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 
 	name := req.Name
 
-	tx, err := d.db.Begin()
+	tx, err := shared.DbBegin(d.db)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -165,7 +156,7 @@ func profilePut(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 
-	tx, err := d.db.Begin()
+	tx, err := shared.DbBegin(d.db)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -180,7 +171,7 @@ func profilePut(d *Daemon, r *http.Request) Response {
 		var i int
 		err = rows.Scan(&i)
 		if err != nil {
-			fmt.Printf("DBERR: profilePut: scan returned error %q\n", err)
+			shared.Debugf("DBERR: profilePut: scan returned error %q\n", err)
 			tx.Rollback()
 			return InternalError(err)
 		}
@@ -189,7 +180,7 @@ func profilePut(d *Daemon, r *http.Request) Response {
 	rows.Close()
 	err = rows.Err()
 	if err != nil {
-		fmt.Printf("DBERR: profilePut: Err returned an error %q\n", err)
+		shared.Debugf("DBERR: profilePut: Err returned an error %q\n", err)
 		tx.Rollback()
 		return InternalError(err)
 	}
@@ -220,29 +211,27 @@ func profilePut(d *Daemon, r *http.Request) Response {
 	return EmptySyncResponse
 }
 
-func profileDelete(d *Daemon, r *http.Request) Response {
-	name := mux.Vars(r)["name"]
-	tx, err := d.db.Begin()
+func dbProfileDelete(db *sql.DB, name string) error {
+	tx, err := shared.DbBegin(db)
 	if err != nil {
-		return InternalError(err)
+		return err
 	}
-	_, err = tx.Exec(`DELETE FROM profiles_config
-			WHERE id IN (SELECT profiles_config.id FROM
-			profiles_config JOIN profiles ON profiles_config.profile_id=profiles.id
-			WHERE profiles.name=?)`, name)
-	if err != nil {
-		tx.Rollback()
-		shared.Debugf("Error deleting profile %s: %s\n", name, err)
-		return InternalError(err)
-	}
-
 	_, err = tx.Exec("DELETE FROM profiles WHERE name=?", name)
 	if err != nil {
 		tx.Rollback()
-		return InternalError(err)
+		return err
 	}
 
 	err = shared.TxCommit(tx)
+
+	return err
+}
+
+// The handler for the delete operation.
+func profileDelete(d *Daemon, r *http.Request) Response {
+	name := mux.Vars(r)["name"]
+	err := dbProfileDelete(d.db, name)
+
 	if err != nil {
 		return InternalError(err)
 	}
