@@ -1,8 +1,5 @@
 package shared
 
-// This package is intended for rendering ContainerState for use between LXC
-// and LXD.
-
 import (
 	"database/sql"
 	"fmt"
@@ -62,19 +59,48 @@ type Device map[string]string
 type Devices map[string]Device
 
 type ContainerState struct {
+	Name            string            `json:"name"`
+	Profiles        []string          `json:"profiles"`
+	Config          map[string]string `json:"config"`
+	ExpandedConfig  map[string]string `json:"expanded_config"`
+	Userdata        []byte            `json:"userdata"`
+	Status          ContainerStatus   `json:"status"`
+	Devices         Devices           `json:"devices"`
+	ExpandedDevices Devices           `json:"expanded_devices"`
+	Ephemeral       bool              `json:"ephemeral"`
+	Log             string            `json:"log"`
+}
+
+/*
+ * BriefContainerState contains a subset of the fields in
+ * ContainerState, namely those which a user may update
+ */
+type BriefContainerState struct {
 	Name      string            `json:"name"`
 	Profiles  []string          `json:"profiles"`
 	Config    map[string]string `json:"config"`
-	Userdata  []byte            `json:"userdata"`
-	Status    ContainerStatus   `json:"status"`
 	Devices   Devices           `json:"devices"`
 	Ephemeral bool              `json:"ephemeral"`
 }
 
+func (c *ContainerState) BriefState() BriefContainerState {
+	retstate := BriefContainerState{Name: c.Name,
+		Profiles:  c.Profiles,
+		Config:    c.Config,
+		Devices:   c.Devices,
+		Ephemeral: c.Ephemeral}
+	return retstate
+}
+
 func (c *ContainerState) State() lxc.State {
 	return lxc.StateMap[c.Status.State]
 }
 
+type ContainerInfo struct {
+	State ContainerState `json:"state"`
+	Snaps []string       `json:"snaps"`
+}
+
 type ContainerAction string
 
 const (
@@ -171,7 +197,7 @@ func ValidDeviceConfig(t, k, v string) bool {
 			return true
 		case "source":
 			return true
-		case "readonly":
+		case "readonly", "optional":
 			return true
 		default:
 			return false
