@@ -22,13 +22,14 @@ import (
 	"path"
 	"path/filepath"
 	"strconv"
+	"strings"
 	"syscall"
 
 	"github.com/gorilla/websocket"
 	"github.com/gosexy/gettext"
 )
 
-// #cgo LDFLAGS: -lutil
+// #cgo LDFLAGS: -lutil -lpthread
 /*
 #include <unistd.h>
 #include <stdlib.h>
@@ -90,7 +91,7 @@ void configure_pty(int fd) {
 	return;
 }
 
-void create_pty(int *master, int *slave) {
+void create_pty(int *master, int *slave, int uid, int gid) {
 	if (openpty(master, slave, NULL, NULL, NULL) < 0) {
 		printf("Failed to openpty: %s\n", strerror(errno));
 		return;
@@ -98,6 +99,15 @@ void create_pty(int *master, int *slave) {
 
 	configure_pty(*master);
 	configure_pty(*slave);
+
+	if (fchown(*slave, uid, gid) < 0) {
+		printf("Warning: error chowning pty to container root\n");
+		printf("Continuing...\n");
+	}
+	if (fchown(*master, uid, gid) < 0) {
+		printf("Warning: error chowning pty to container root\n");
+		printf("Continuing...\n");
+	}
 }
 
 void create_pipe(int *master, int *slave) {
@@ -115,11 +125,13 @@ void create_pipe(int *master, int *slave) {
 */
 import "C"
 
-func OpenPty() (master *os.File, slave *os.File, err error) {
+func OpenPty(uid, gid int) (master *os.File, slave *os.File, err error) {
 	fd_master := C.int(-1)
 	fd_slave := C.int(-1)
+	rootUid := C.int(uid)
+	rootGid := C.int(gid)
 
-	C.create_pty(&fd_master, &fd_slave)
+	C.create_pty(&fd_master, &fd_slave, rootUid, rootGid)
 
 	if fd_master == -1 || fd_slave == -1 {
 		return nil, nil, errors.New("Failed to create a new pts pair")
@@ -305,7 +317,7 @@ func WebsocketRecvStream(w io.WriteCloser, conn *websocket.Conn) chan bool {
 // WebsocketMirror allows mirroring a reader to a websocket and taking the
 // result and writing it to a writer.
 func WebsocketMirror(conn *websocket.Conn, w io.WriteCloser, r io.Reader) chan bool {
-	done := make(chan bool, 1)
+	done := make(chan bool, 2)
 	go func(conn *websocket.Conn, w io.WriteCloser) {
 		for {
 			mt, r, err := conn.NextReader()
@@ -563,6 +575,38 @@ func CopyFile(dest string, source string) error {
 	return err
 }
 
+/* Some interesting filesystems */
+const (
+	tmpfsSuperMagic = 0x01021994
+	ext4SuperMagic  = 0xEF53
+	xfsSuperMagic   = 0x58465342
+	nfsSuperMagic   = 0x6969
+)
+
+func GetFilesystem(path string) (string, error) {
+	fs := syscall.Statfs_t{}
+
+	err := syscall.Statfs(path, &fs)
+	if err != nil {
+		return "", err
+	}
+
+	switch fs.Type {
+	case btrfsSuperMagic:
+		return "btrfs", nil
+	case tmpfsSuperMagic:
+		return "tmpfs", nil
+	case ext4SuperMagic:
+		return "ext4", nil
+	case xfsSuperMagic:
+		return "xfs", nil
+	case nfsSuperMagic:
+		return "nfs", nil
+	default:
+		return string(fs.Type), nil
+	}
+}
+
 type BytesReadCloser struct {
 	Buf *bytes.Buffer
 }
@@ -575,3 +619,36 @@ func (r BytesReadCloser) Close() error {
 	/* no-op since we're in memory */
 	return nil
 }
+
+func IsSnapshot(name string) bool {
+	return strings.Contains(name, "/")
+}
+
+func ReadLastNLines(f *os.File, lines int) (string, error) {
+	if lines <= 0 {
+		return "", fmt.Errorf("invalid line count")
+	}
+
+	stat, err := f.Stat()
+	if err != nil {
+		return "", err
+	}
+
+	data, err := syscall.Mmap(int(f.Fd()), 0, int(stat.Size()), syscall.PROT_READ, syscall.MAP_SHARED)
+	if err != nil {
+		return "", err
+	}
+	defer syscall.Munmap(data)
+
+	for i := len(data) - 1; i >= 0; i-- {
+		if data[i] == '\n' {
+			lines--
+		}
+
+		if lines < 0 {
+			return string(data[i+1 : len(data)]), nil
+		}
+	}
+
+	return string(data), nil
+}
